/************************************/
/* Gloobus-preview configuration    */
/*                                  */
/************************************/

#include "gloobus-preview-config.h"
#include <gconf/gconf-client.h>

//============== CONSTRUCTOR ==============//
gloobusConfig::gloobusConfig(void)
    :m_taskbar      (true)
    ,m_ontop        (false)
    ,m_focus        (false)
    ,m_winbar_layout(true)
    ,cli_debug      (false)
    ,cli_action     (GP_ACTION_PREVIEW)
    ,cli_target(0)
{}

//============== DESTRUCTOR ==============//
gloobusConfig::~gloobusConfig(void){}

//============== LOAD GCONF WIN LAYOUT ==============//
bool gloobusConfig::get_gconf_win_layout()
{
    GConfClient* client_;
    GError* error = NULL;
    client_ = gconf_client_get_default();
    if (!client_) {
        g_debug("Unable to create a gconf client");
        return false;
    }
    gchar* gconf_value = gconf_client_get_string(client_, "/apps/metacity/general/button_layout", &error);
    if (!gconf_value) {
        return false;
    }
    if (int(string(gconf_value).find("close")) - int(string(gconf_value).find(":")) > 0)
        m_winbar_layout = false;
    else
        m_winbar_layout = true;
        
    return true;
}

//============== LOAD CONFIG ==============//
bool gloobusConfig::load_config()
{
    string home = getenv("HOME");
    string configFile = home + "/.config/gloobus/gloobus-preview.cfg";

    GKeyFile * keyFile = g_key_file_new ( );
    g_debug("Loading config file %s", configFile.c_str());

    if(g_key_file_load_from_file ( keyFile,configFile.c_str(),(GKeyFileFlags)0,NULL ) )
    {
      
        m_ontop 	= (bool)g_key_file_get_integer   		( keyFile,"Main","ontop",	NULL );
        m_focus 	= (bool)g_key_file_get_integer   		( keyFile,"Main","focus",	NULL );
        if (!get_gconf_win_layout())
            m_winbar_layout = (bool)g_key_file_get_integer 	( keyFile,"Main","winbar_layout",	NULL );
        g_key_file_free										( keyFile );

		if(m_ontop || m_focus)
			  m_taskbar = false;

        return true;
    }
    else
    {
        g_warning("Config file couldn't be readed");
        return false;
    }
}

// =============== PARSE COMMAND LINE ARGUMENTS =============== //
void gloobusConfig::parse_args(int argc, char **argv) {
    for (int i=1 ; i < argc ; i++) {
        if( !strcmp(argv[i] , "--show-supported-filetypes") ) {
            cli_action = GP_ACTION_FILETYPES;
        } else if( !strcmp(argv[i], "--help") or !strcmp(argv[i], "-h")) {
            cli_action = GP_ACTION_HELP;
        } else if( !strcmp(argv[i], "--debug") or !strcmp(argv[i], "-d")) {
            cli_debug = true;
        } else {
            cli_target = argv[i];
        }
    }
}

//=============== GET METHODS =============== //
bool	gloobusConfig::get_taskbar			(void){return m_taskbar;}
bool	gloobusConfig::get_ontop			(void){return m_ontop;}
bool	gloobusConfig::get_focus			(void){return m_focus;}
bool	gloobusConfig::get_winbar_layout	(void){return m_winbar_layout;}

bool	gloobusConfig::get_cli_debug		(void){return cli_debug;}
char *	gloobusConfig::get_cli_target		(void){return cli_target;}
gp_action gloobusConfig::get_cli_action		(void){return cli_action;}
