// ========================== TEXT INTERFACE ======================== //
// Type: document				 							   //
// =================================================================== //

#ifndef _IDOCUMENT_
#define _IDOCUMENT_

#include <map>
#include <string>
#include <vector>
#include <gtk/gtk.h>

#include "gloobus-preview-interface-main.h"

#define PAGE_SPACING 5
#define PAGE_PRELOAD 5


using namespace std;

class iDocument : public interface {
	protected:
		GtkWidget		*	m_scwin;  //Scolled window to store the document pages
		GtkAdjustment	*	m_hadj; //To move the horizontal scroll
		GtkAdjustment	*	m_vadj;
		GtkWidget		*	m_vBox;   //Vertical Box to put the pages
		gdouble				m_upper;
		gint				n_pages;
		gint				current_page;
		gdouble				current_position;
		GtkWidget		**	pages;
		GtkWidget		**	page_containers;
		gint			*	page_heights;
		GtkWidget		*	info_label;
		PangoAttrList	*	info_attrs;
		string				file_info;
		gint				page_width;
		gint				page_height;
		gint				visible_width;
		gint				visible_height;
		guint				loader_id;
		gpointer		*	self_ptr;	// we need this to avoid segmentation
										// fault when iDocument is destroyed
										// and page_loader is still running

	public:
							iDocument();
		virtual 			~iDocument();

		//Virtual Functions implemented from interface
		virtual int      get_width        ( void );
		virtual int      get_height       ( void );
		virtual	void     draw             ( GtkContainer * container );
		virtual	void     draw_toolbar     ( GtkWidget * container );
		virtual bool     load             ( void );
		virtual	bool	 load_doc	      ( void )	= 0;
		virtual void     end              ( void );
		virtual bool     can_fullscreen   ( void );//Returns true if the plugin can do fullscreen
		virtual void     draw_fullscreen  ( GtkContainer * container );
		virtual void     draw_unfullscreen( GtkContainer * container );

				float		get_scale			( int x, int y );
				void		draw_doc			( GtkContainer * container); 
				void		draw_file_info		( GtkContainer * container);
		static	gboolean	page_loader			(gpointer);
		static	gboolean	expose_event		(GtkWidget *, GdkEventExpose *, gpointer);
		static	void		check_scrollbar		(GtkAdjustment *, gpointer);
				void		set_page_info		(int);
				void		set_page_widget		(GtkWidget*, GtkWidget*,bool=true);
		virtual	void		key_press_event		(int);
		
		//These next virtual functions musth be implemented for iDocument plugins
		virtual	int			get_n_pages			( void )	= 0;
		virtual	int			get_page_width		( void )	= 0;
		virtual	int			get_page_height		( void )	= 0;
		virtual	GdkPixbuf*	get_page_pixbuf		( int page)	= 0;
		virtual	GtkWidget*	get_page			( int )		= 0;
		virtual	void		end_plugin			( void )	= 0;
};

//==============  CLASS FACTORIES DEFINITIONS================================= //
typedef iDocument* create_t_document ();						//CREATE PLUGIN
typedef void destroy_t_document  (iDocument*);						//DESTROY PLUGIN
	
extern map<string, create_t_document *, less<string> > factory_document;		//MAP WITH NAME AND CONSTRUCTOR
extern map<string, destroy_t_document *, less<string> > factoryDestroy_document;	//MAP WITH NAME AND DESTRUCTOR
extern vector<plugin_filetype> pluginVector;						//VECTOR TO STORE WICH FILETYPE READ EACH PLUGIN


#endif
