#include "gloobus-preview-interface-image.h"

iImage::iImage()
	:m_pixbuf(0)
	,m_pixbufFull(0)
	,m_image(0)
	,m_image_container(0)
{}

// ========================= DESTRUCTOR ================================== //
iImage::~iImage()
{
	g_debug("Destroying iImage");
	if(m_image)
		gtk_widget_destroy(m_image);
	if(m_image_container)
		gtk_widget_destroy(m_image_container);
	if(m_pixbuf)
		g_object_unref(m_pixbuf);
	if(m_pixbufFull)
		g_object_unref(m_pixbufFull);
}
//=========================  END ========================================= //
void iImage::end()
{
	g_debug("Ending iImage");
}
// ========================= CONSTRUCTOR ================================= //
bool iImage::load()
{
	g_debug("Loading iImage");
	GdkPixbuf* pixbuf = get_pixbuf();
	if(pixbuf == NULL) {
		m_error = g_error_new(g_quark_from_string ("iImage"), 101,
			_("Error loading image"));
		return false;
	}
    m_pixbufFull = scale_pixbuf(pixbuf, get_screen_width(), get_screen_height());
    m_pixbuf 	 = scale_pixbuf(m_pixbufFull, get_screen_width()-150,get_screen_height()-200);
    g_object_unref(pixbuf);

    return true;
};

// =========================== GET SIZE (W)================================= //
int  iImage::get_width()
{
	//g_debug("Width: %d\n", gdk_pixbuf_get_width(m_pixbuf));
	if(gdk_pixbuf_get_width(m_pixbuf) < 300)
	{
	 	return 300;
	}
	return gdk_pixbuf_get_width(m_pixbuf);
};

// =========================== GET SIZE (H)================================= //
int  iImage::get_height()
{
	//g_debug("Height: %d\n",gdk_pixbuf_get_height(m_pixbuf));
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)
	{
		return 300;
	}
	return gdk_pixbuf_get_height(m_pixbuf) + 15;		//FIXME: Why do I have to add this +20?
};

// =========================== DRAW IMAGE ================================= //
void iImage::draw_image(GtkContainer * container)
{

	m_image 			= gtk_image_new_from_pixbuf (m_pixbuf);
	m_image_container	= gtk_fixed_new ();
	int x = SHADOW_WIDTH;			//Default values are at top left
	int y = HEADER_HEIGHT + SHADOW_WIDTH;			//and 20 from top (For the header)

	if(gdk_pixbuf_get_width(m_pixbuf) < 300)		//Minum width
	{
		x = (150 + SHADOW_WIDTH ) - gdk_pixbuf_get_width(m_pixbuf)/2;
	}
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)		//Minium height
	{
		y = (150 + SHADOW_WIDTH + HEADER_HEIGHT) - gdk_pixbuf_get_height(m_pixbuf)/2;
	}


	gtk_fixed_put 	  (GTK_FIXED(m_image_container),m_image,x,y);
	gtk_container_add (GTK_CONTAINER(container),m_image_container);
	gtk_widget_show   (m_image_container);
	gtk_widget_show   (m_image);

};

// ========================= DRAW IMAGE INFO ===================== //
void iImage::draw_file_info(GtkContainer * container)
{
	draw_properties ( container, get_file_info_string(), 25 + SHADOW_WIDTH, get_height() + SHADOW_WIDTH + HEADER_HEIGHT - 10 );
}

// ========================= DRAW ===================== //
// Virtual function that draws what plugin wants
void iImage::draw( GtkContainer * container )
{
	draw_file_info	( GTK_CONTAINER(container ));
	draw_image		( GTK_CONTAINER(container ));
	draw_controls	( GTK_CONTAINER(container ));
}

// ======================== FULLSCREEN ==================== //
bool iImage::can_fullscreen  ( void ){	return true;} 
void iImage::draw_fullscreen ( GtkContainer * container )
{
	int x,y;
	x = get_screen_width()/2  - gdk_pixbuf_get_width(m_pixbufFull)/2;
	y = get_screen_height()/2 - gdk_pixbuf_get_height(m_pixbufFull)/2;
	
	g_debug("Fullscreen Width:%i", gdk_pixbuf_get_width(m_pixbufFull));
	g_debug("Fullscreen Height:%i",gdk_pixbuf_get_height(m_pixbufFull));

	gtk_image_set_from_pixbuf	(GTK_IMAGE(m_image), m_pixbufFull);
	gtk_fixed_move 	  			(GTK_FIXED(m_image_container),m_image,x,y);

	show_fullscreen_controls(container);
	
}
// ======================== UN FULLSCREEN ==================== //
void iImage::draw_unfullscreen ( GtkContainer * container )
{
	int x = SHADOW_WIDTH;							//Default values are at top left
	int y = HEADER_HEIGHT + SHADOW_WIDTH;			//and 20 from top (For the header)

	if(gdk_pixbuf_get_width(m_pixbuf) < 300)		//Minum width
	{
		x = (150 + SHADOW_WIDTH ) - gdk_pixbuf_get_width(m_pixbuf)/2;
	}
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)		//Minium height
	{
		y = (150 + SHADOW_WIDTH + HEADER_HEIGHT) - gdk_pixbuf_get_height(m_pixbuf)/2;
	}
	
	gtk_image_set_from_pixbuf	(GTK_IMAGE(m_image), m_pixbuf);
	gtk_fixed_move 	  			(GTK_FIXED(m_image_container),m_image,x,y);

	show_unfullscreen_controls(container);
	
}

// ============================ DRAW FLOATING CONTROLS ================================ //
void iImage::draw_toolbar	( GtkWidget * container )
{
    g_debug("iImage toolbar buttons\n");

	draw_default_toolbar(container);
}

