#include "gloobus-preview-interface-list.h"

// ================m_scwin========= DESTRUCTOR =================================== //
iList::iList()
	:m_pixbuf(0)
	,m_reflection(0)
	,n_items(0)
	,m_store(0)
	,m_scwin(0)
	,m_tree_view(0)
	,loader_id(0)
	,self_ptr(0)
{
	g_debug("Creating iList");
	self_ptr = new gpointer;
	*self_ptr = (gpointer)this;
}
iList::~iList()
{
	g_debug("Destroying iList");
	if(m_pixbuf)
		g_object_unref( m_pixbuf );
	if(m_reflection)
		g_object_unref( m_reflection );
	if(m_store)
		g_object_unref( m_store );
}
//============================= END =========================================== //
void iList::end()
{
	g_debug("Ending iList");
	if(loader_id)
		g_source_remove(loader_id);
	if(*self_ptr)
		*self_ptr = 0;
	else
		delete self_ptr;

	if(m_scwin)
		gtk_widget_destroy(m_scwin);
	g_debug("ended");
}
// ========================= CONSTRUCTOR ================================= //
bool iList::load()
{
	g_debug("Loading iList");
	GdkPixbuf* temp = get_pixbuf();
	m_pixbuf 	 = scale_pixbuf(temp ,ICON_SIZE,ICON_SIZE);
	m_reflection = pixbuf_reflection(m_pixbuf);		//Creates the reflection Pixbuf
	g_object_unref(temp);

	m_scwin = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(m_scwin),
		GTK_POLICY_AUTOMATIC ,GTK_POLICY_AUTOMATIC );
	gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW(m_scwin),
		GTK_SHADOW_NONE);
	gtk_widget_set_size_request( m_scwin , 500 - 2 * SHADOW_WIDTH - 18, 280);
	//FIXME: shouldn't use 500. Should get window width.

	//n_items = get_n_items();
	m_store = get_list_store();

	GtkTreeViewColumn *col;
	GtkCellRenderer   *renderer;

	m_tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(m_store));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(m_tree_view), true);

	for(int k=0; k< get_n_columns(); k++) {
		if(is_column_pixbuf(k)) {
			renderer = gtk_cell_renderer_pixbuf_new( );                                                
			col = gtk_tree_view_column_new_with_attributes (
				get_column_title(k).c_str(), renderer, "pixbuf", k, NULL);
		} else {
			renderer = gtk_cell_renderer_text_new( );
			col = gtk_tree_view_column_new_with_attributes(
				get_column_title(k).c_str(), renderer, "markup", k, NULL);
			gtk_tree_view_column_set_sort_column_id  (col,k);
		}
		gtk_tree_view_append_column( GTK_TREE_VIEW(m_tree_view), col);
	}

	gtk_container_add( GTK_CONTAINER(m_scwin), m_tree_view);

	loader_id = g_idle_add(&file_loader, self_ptr);

	return true;
};

// =========================== GET SIZE (W)================================= //
int  iList::get_width()
{
	return 500;
};

// =========================== GET SIZE (H)================================= //
int  iList::get_height()
{
	return 500;
};

// =========================== DRAW ICON ================================= //
void iList::draw_icon(GtkContainer * container)
{
	GtkWidget *image 				= gtk_image_new_from_pixbuf (m_pixbuf);
	GtkWidget *image_container		= gtk_fixed_new ();
	GtkWidget *reflection  			= gtk_image_new_from_pixbuf (m_reflection);
	GtkWidget *reflection_container	= gtk_fixed_new ();

	int x = SHADOW_WIDTH;							//Default values are at top left
	int y = HEADER_HEIGHT + SHADOW_WIDTH;			//and 20 from top (For the header)

	if(gdk_pixbuf_get_width(m_pixbuf) < 300)		//Minum width
	{
		x = 150-gdk_pixbuf_get_width(m_pixbuf)/2;
	}
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)		//Minium height
	{
		y = 150-gdk_pixbuf_get_height(m_pixbuf)/2;
	}


	gtk_fixed_put 	  	(GTK_FIXED(image_container),image,50+SHADOW_WIDTH, 26 + SHADOW_WIDTH + HEADER_HEIGHT);
	gtk_fixed_put 	  	(GTK_FIXED(reflection_container),reflection,50+SHADOW_WIDTH, 26 + SHADOW_WIDTH + HEADER_HEIGHT + gdk_pixbuf_get_height(m_pixbuf));
	gtk_container_add 	(GTK_CONTAINER(container),image_container);
	gtk_container_add 	(GTK_CONTAINER(container),reflection_container);
	gtk_widget_show   	(image_container);
	gtk_widget_show   	(image);
	gtk_widget_show	 	(reflection_container);
	gtk_widget_show	 	(reflection);
}

// =========================== DRAW LIST ================================= //
void iList::draw_list(GtkContainer * container)
{
	gtk_fixed_put(GTK_FIXED(container), m_scwin, 24 + SHADOW_WIDTH,
		240 + SHADOW_WIDTH + HEADER_HEIGHT);
	gtk_widget_show_all ( m_scwin );
};

// ========================= DRAW IMAGE INFO ===================== //
void iList::draw_file_info(GtkContainer * container)
{
	draw_properties ( container, get_file_info_string(), 285 + SHADOW_WIDTH,80 + SHADOW_WIDTH);
}

// ========================= DRAW ===================== //
// Virtual function that draws what plugin wants
void iList::draw( GtkContainer * container )
{
	draw_icon		( GTK_CONTAINER(container ));
	draw_list		( GTK_CONTAINER(container ));
	draw_file_info	( GTK_CONTAINER(container ));
}

// ====================== FULLSCREEN ================= //
bool iList::can_fullscreen  ( void ){ return false;}						//Returns true if the plugin can do fullscreen
void iList::draw_fullscreen ( GtkContainer * container )
{
}
void iList::draw_unfullscreen ( GtkContainer * container )
{
}

// ============================ DRAW FLOATING CONTROLS ================================ //
void iList::draw_toolbar	( GtkWidget * container )
{
    g_debug("iList toolbar buttons");
	draw_default_toolbar(container);
}


gboolean iList::file_loader(gpointer ptr) {
	g_debug("File loader started");
	if(*((iList**)ptr) == 0) {
		delete (iList**)ptr;
		return false;
	}
	iList* list = *((iList**)(ptr));
	GtkTreeIter iter;

	while(list->has_items()) {
		g_debug("Adding %d", list->n_items);
		int i = list->n_items;
		gtk_list_store_append(list->m_store, &iter);
		for ( int j = 0 ; j < list->get_n_columns() ; j++) {
			if( list->is_column_pixbuf(j) ) {
				gtk_list_store_set ( list->m_store, &iter, j,
					list->get_item_pixbuf(i,j),-1);
			} else {
				gtk_list_store_set ( list->m_store, &iter, j,
					list->get_item(i,j).c_str(),-1);
			}
		}
		list->n_items++;
		while (gtk_events_pending ())
			gtk_main_iteration ();
		if(*((iList**)ptr) == 0) {
			delete (iList**)ptr;
			return false;
		}
	}
	if(list->n_items == 0) {
		if(!list->m_error) {
			list->m_error = g_error_new(g_quark_from_string ("iList"), 901,
			_("Error listing files"));
		}
		Gloobus::instance()->end_file_and_error();
	}
	return false;
}
