#include "gloobus-preview-interface-main.h"

interface::interface()
	:m_gfile(0)
	,m_file_info(0)
	,m_info(0)
	,m_error(0)
	,m_fullscreen_button(0)
	,m_defaultapp_button(0)
	,m_fullscreen_box(0)
	,m_defaultapp_box(0)
	,m_hbox(0)
{
	g_debug("Creating interface");
}

interface::~interface() {
	g_debug("Destroying interface");
	if(m_gfile)
		g_object_unref(m_gfile);
	if(m_file_info)
		g_object_unref(m_file_info);
	if(m_error)
		g_error_free(m_error);
	if(m_info)
		gtk_widget_destroy(m_info);
	if (m_fullscreen_button)
		gtk_widget_destroy(m_fullscreen_button);
	if(m_defaultapp_button)
		gtk_widget_destroy(m_defaultapp_button);
	if(m_fullscreen_box)
		gtk_widget_destroy(m_fullscreen_box);
	if(m_defaultapp_box)
		gtk_widget_destroy(m_defaultapp_box);
	if(m_hbox)
		gtk_widget_destroy(m_hbox);
}

// ================================= SET FILE NAME ============================ //
// @param filename is the path to the file (not URI)
// @returns void
//
// It sets the member variable filename, gfile and sets the basic information
bool interface::set_filename( string filename )	
{
	m_filename = filename; 
	m_gfile    = g_file_new_for_path (filename.c_str () );

	m_fullscreen_event = false;
	set_file_info();
	if( m_error )
		return false;
	return true;
}
// ================================= SET FILE INFO ========================== //
void interface::set_file_info ()
{

	//m_file_info		 = g_file_info_new	 ();
	m_file_info 	 = g_file_query_info(  m_gfile,
		G_FILE_ATTRIBUTE_STANDARD_TYPE ","
		G_FILE_ATTRIBUTE_STANDARD_ICON ","
		G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
		G_FILE_ATTRIBUTE_STANDARD_SIZE  ","
		G_FILE_ATTRIBUTE_THUMBNAIL_PATH ","
		G_FILE_ATTRIBUTE_PREVIEW_ICON,
		G_FILE_QUERY_INFO_NONE, NULL , &m_error);

	if( m_error )
		return;

	m_filetype = g_file_info_get_attribute_string ( m_file_info,
		"standard::content-type" );

	g_debug(CLI_COLOR_GREEN "FILE INFO" CLI_COLOR_RESET);
	g_debug("Screen size: %ix%i", get_screen_width(),get_screen_height());
	
	char * uri = g_file_get_uri ( m_gfile); // FIXME: why  doing this if
	g_debug("URI:   %s",uri);              //        debug is not rquired
	g_free(uri);
	
	g_debug("THUMB: %s",g_file_info_get_attribute_byte_string( m_file_info,
		"thumbnail::path" ));
	g_debug("Type:  %s" ,m_filetype.c_str());
	g_debug("Size:  %s",get_file_size_string(m_filename).c_str());
}


// ================================= GET SAFE TYPE ========================== //
string interface::get_safe_type	(bool replace_slash )
{
	string mime = m_filetype;
	mime = mime.substr(0,mime.find(";"));
	mime = mime.substr(0,mime.find(","));

	if(mime == "x-directory/normal")		{ mime = "folder";}

	if(replace_slash)
	{
		if(mime.find("/") != string::npos)
			mime.replace(mime.find("/"),1,"-");
	}

	return mime;
}

// ================================ TRASH FILE ============================== //
void interface::trash_file	( void )
{	
	g_file_trash(m_gfile,NULL,NULL);
}
// ================================ GET FILE SIZE =========================== //
string interface::get_file_size_string(string filename)
{
    GFile 	  * file = g_file_new_for_path( filename.c_str() );
	GFileInfo * info;
	info        = g_file_query_info                 (file, "*" ,G_FILE_QUERY_INFO_NONE, NULL , NULL);
	string mime = g_file_info_get_attribute_string  ( info,	"standard::content-type" );

    if(mime== "inode/directory")
    {
        return _("Directory");
    }
   
	char * size = g_format_size_for_display (get_file_size(filename));
	string result(size);
	g_free(size);
	return result;

}
// ================================ GET FILE SIZE ============================== //
double interface::get_file_size	( string filename)
{
	
	GFile 	  * file = g_file_new_for_path( filename.c_str() );
	GFileInfo * info;
	info        = g_file_query_info                 (file, "*" ,G_FILE_QUERY_INFO_NONE, NULL , NULL);
	string mime = g_file_info_get_attribute_string  ( info,	"standard::content-type" );
	
	double size;
    
    size = g_file_info_get_attribute_uint64 ( info, "standard::size" );
    if(mime== "inode/directory")
    {
    	return -1;
    }
 
    g_object_unref(file);
    g_object_unref(info);
	return size;

}

// ================================= DRAW FILE INFO ============================ //
void interface::draw_file_info( GtkContainer * container)
{
	draw_properties(container , get_file_info_string (), 20 + SHADOW_WIDTH , get_height() + SHADOW_WIDTH - 20);
}

// ================================= DRAW PROPERTIES ============================ //
void interface::draw_properties( GtkContainer * container , string text, int x, int y)
{

	PangoAttrList* at = pango_attr_list_new();
	pango_attr_list_insert(at,pango_attr_foreground_new(65000,65000,65000));
	pango_attr_list_insert(at,pango_attr_weight_new(PANGO_WEIGHT_NORMAL));

	//GtkWidget * label = gtk_label_new(text.c_str());
	m_info = gtk_label_new(text.c_str());
	
						g_debug("MAX LETTERS : %i",(get_width()-SHADOW_WIDTH*2-10-x)/6);
			            gtk_label_set_max_width_chars (GTK_LABEL(m_info), (get_width()-SHADOW_WIDTH*2-10-x)/6 );
	gtk_label_set_attributes(GTK_LABEL(m_info),at);

	GtkWidget *label_container = gtk_fixed_new ();
	gtk_fixed_put(GTK_FIXED(container),m_info, x ,y);
	gtk_container_add (GTK_CONTAINER(container),label_container);

	gtk_widget_show	  (m_info);
	gtk_widget_show	  (label_container);
}

// ================================= SHOW INFO ============================ //
void interface::show_info	( void )
{
	gtk_widget_show(m_info);
	gtk_widget_show(m_hbox);
}
// ================================= HIDE INFO ============================ //
void interface::hide_info	( void )
{
	gtk_widget_hide(m_info);
	gtk_widget_hide(m_hbox);
}

// ================================= SHOW FULLSCREEN CONTROLS ============================ //
void interface::show_fullscreen_controls	(GtkContainer * container)
{
	string fullbuttonstr	   = (string)PACKAGE_DATA_PATH + "/view-restore.svg";
	gtk_image_set_from_file		(GTK_IMAGE(m_fullscreen_button), fullbuttonstr.c_str());
	

//	gtk_fixed_move 	  			(GTK_FIXED(container),m_hbox,get_screen_width()/2 - 32 ,get_screen_height() - 97);
	gtk_widget_hide				(m_hbox);
	gtk_widget_hide(m_info);
}
// ================================= HIDE FULLSCREEN CONTROLS ============================ //
void interface::show_unfullscreen_controls	(GtkContainer * container)
{
	string fullbuttonstr	   = (string)PACKAGE_DATA_PATH + "/view-fullscreen.svg";
	gtk_image_set_from_file		(GTK_IMAGE(m_fullscreen_button), fullbuttonstr.c_str());
	
	gtk_fixed_move 	  			(GTK_FIXED(container),m_hbox,get_width()/2 - 25 ,get_height()+30);
	show_info();
}

// ================================= DRAW CONTROLS ======================== //
void interface::draw_controls ( GtkContainer * container )
{
	int displace = 5;

	//Fullscreen button
	if(can_fullscreen())
	{
		string fullbuttonstr	= (string)PACKAGE_DATA_PATH + "/view-fullscreen.svg";
		m_fullscreen_button		= gtk_image_new_from_file(fullbuttonstr.c_str());
		m_fullscreen_box		= gtk_event_box_new();
								  gtk_event_box_set_visible_window(GTK_EVENT_BOX(m_fullscreen_box),false);
								  gtk_container_add(GTK_CONTAINER(m_fullscreen_box),m_fullscreen_button);
		displace+=15;
	}

	//Launch Default App buton
	string defaultappstr	= (string)PACKAGE_DATA_PATH + "/document-open.svg";
	m_defaultapp_button		= gtk_image_new_from_file(defaultappstr.c_str());
	m_defaultapp_box		= gtk_event_box_new();
							  gtk_event_box_set_visible_window(GTK_EVENT_BOX(m_defaultapp_box),false);
							  gtk_container_add(GTK_CONTAINER(m_defaultapp_box),m_defaultapp_button);


	m_hbox 	= gtk_hbox_new    (false, 3);
	gtk_box_pack_start 		( GTK_BOX (m_hbox),   m_defaultapp_box	, 		false, 	false, 	2);

	if(can_fullscreen())
	{
		gtk_box_pack_start 	( GTK_BOX (m_hbox),   m_fullscreen_box	, 		false, 	false, 	2);
	}
	
	gtk_fixed_put			( GTK_FIXED(container), m_hbox, get_width()/2 - displace , get_height() + 32 );

	if(get_width()>300)	//Only show it when there is enough space
		gtk_widget_show_all	( m_hbox );

	if(can_fullscreen()) {
		g_signal_connect 	( G_OBJECT (m_fullscreen_box), 	"button-press-event",   G_CALLBACK (&interface::fullscreen_trick), this);
	}
	g_signal_connect 	( G_OBJECT (m_defaultapp_box), 	"button-press-event",   G_CALLBACK (&interface::defaultapp_trick), this);
}

// ================================= DRAW FLOATING CONTROLS ====================== //
void interface::draw_default_toolbar	( GtkWidget * container )
{

	g_debug("Adding toolbar buttons");

	//Fullscreen button
	string fullbuttonstr			= 	(string)PACKAGE_DATA_PATH + "/view-restore.svg";
	GtkWidget* fullscreen_button	= 	gtk_image_new_from_file(fullbuttonstr.c_str());
	GtkWidget* fullscreen_box		= 	gtk_event_box_new();
							  			gtk_event_box_set_visible_window(GTK_EVENT_BOX(fullscreen_box),false);
							  			gtk_container_add(GTK_CONTAINER(fullscreen_box),fullscreen_button);
	
	//Launch Default App buton
	string defaultappstr			= 	(string)PACKAGE_DATA_PATH + "/document-open.svg";
	GtkWidget* defaultapp_button	= 	gtk_image_new_from_file(defaultappstr.c_str());
	GtkWidget* defaultapp_box		= 	gtk_event_box_new();
							  			gtk_event_box_set_visible_window(GTK_EVENT_BOX(defaultapp_box),false);
							  			gtk_container_add(GTK_CONTAINER(defaultapp_box),defaultapp_button);

	GtkWidget *vbox		= gtk_vbox_new	  (false, 3);
	GtkWidget *hbox 	= gtk_hbox_new    (false, 3);
	
	gtk_box_pack_start 		( GTK_BOX (hbox),   gtk_label_new(""), 		true, 	true, 	2);
	gtk_box_pack_start 		( GTK_BOX (hbox),   defaultapp_box	, 		false, 	false, 	2);
	gtk_box_pack_start 		( GTK_BOX (hbox),   fullscreen_box	, 		false, 	false, 	2);
	gtk_box_pack_start 		( GTK_BOX (hbox),   gtk_label_new(""), 		true, 	true, 	2);	

	gtk_box_pack_start 		( GTK_BOX (vbox),   gtk_label_new(""), 		false, 	false, 	2);	
	gtk_box_pack_start 		( GTK_BOX (vbox),   hbox, 					false, 	false, 	2);	


	g_signal_connect 	( G_OBJECT (fullscreen_box), 	"button-press-event",   G_CALLBACK (&interface::fullscreen_trick), this);
	g_signal_connect 	( G_OBJECT (defaultapp_box), 	"button-press-event",   G_CALLBACK (&interface::defaultapp_trick), this);

	//gtk_box_pack_start ( GTK_BOX(container), hbox, false,false,2);
	gtk_container_add ( GTK_CONTAINER(container), vbox);

	gtk_widget_show_all(container);
}
// ================================= GET FILE INFO STRING ====================== //
string interface::get_file_info_string ( void )
{

	string text;
	char * desc = g_content_type_get_description(m_filetype.c_str());
	
	text = "";
	text = text + _("Type: ") + desc + "\n\n";
	text = text + _("Size: ") + get_file_size_string(m_filename);

	g_free(desc);
	
	return text;

}

// ================================= GET SCREEN SIZE ============================ //
int interface::get_screen_width ( void )
{
	GdkScreen *screen = gdk_screen_get_default ();
	return gdk_screen_get_width(screen);
}
int interface::get_screen_height( void )
{
	GdkScreen *screen = gdk_screen_get_default ();
	return gdk_screen_get_height(screen);
}


//==================================== PIXBUF FUNCTIONS ===========================================//

// ================================= GET ICON ============================ //
GdkPixbuf * interface::get_icon ( void )
{

    GIcon *icon;
    GtkIconInfo *icon_info;
    GtkIconTheme *icon_theme;
    GdkPixbuf* temp;

    icon_theme = gtk_icon_theme_get_default();
    icon = g_file_info_get_icon(m_file_info);
    icon_info = gtk_icon_theme_lookup_by_gicon(
                    icon_theme,
                    icon,
                    200,    /* icon size */
                   GtkIconLookupFlags(GTK_ICON_LOOKUP_USE_BUILTIN | 
                   					  GTK_ICON_LOOKUP_GENERIC_FALLBACK | 
                   					  GTK_ICON_LOOKUP_FORCE_SIZE));

    temp = gtk_icon_info_load_icon(icon_info, NULL);

    return temp;
}

//============================== CREATE REFLECTION ========================= //
GdkPixbuf * interface::pixbuf_reflection	( GdkPixbuf * pixbuf )
{
	/*
	REFLECTION

	 0 = transparent, 255=opac
	 
		  Fliped Pixbuf
		    0 --------------  initial_opacity
		    1 --------------  initial_opacity - (1*constant)
		    2 --------------  initial_opacity - (2*constant)
		    3 --------------  initial_opacity - (3*constant)
		    4 --------------  initial_opacity - (4*constant)
	*/ 
	GdkPixbuf * temp;
	temp = gdk_pixbuf_flip	(pixbuf,false);
	int x 	= gdk_pixbuf_get_width	(temp);
	int y 	= gdk_pixbuf_get_height	(temp);
	int initial_opacity = 77;

	int width, height, rowstride, n_channels;
	guchar *pixels, *p;
	n_channels = gdk_pixbuf_get_n_channels 		( temp );
	g_assert (gdk_pixbuf_get_colorspace 		( temp ) == GDK_COLORSPACE_RGB );
	g_assert (gdk_pixbuf_get_bits_per_sample 	( temp ) == 8);
	g_assert (gdk_pixbuf_get_has_alpha 			( temp ));
	g_assert (n_channels == 4);
	width = gdk_pixbuf_get_width 				( temp );
	height = gdk_pixbuf_get_height 				( temp );
	//g_assert (x >= 0 && x < width);
	//g_assert (y >= 0 && y < height);
	rowstride 	= gdk_pixbuf_get_rowstride 		( temp );
	pixels 		= gdk_pixbuf_get_pixels 		( temp );
	//p 		= pixels + y * rowstride + x * n_channels;



	for(int i=0; i<x; i++)
	{
		for(int j=0; j<y; j++)
		{
			//TODO: Add the set_opacity fucntions here!!
			int less = j*1.3;
			if (less > initial_opacity) less = initial_opacity;
			//set_pixel_opacity(temp,i,j,initial_opacity-less);
			p = pixels + j * rowstride + i * n_channels;
			guchar temp = 255 - (initial_opacity-less);				//initial_opacity-less = alpha
			//guchar temp2 = p[3] - temp;
	
			if		(temp >= p[3]) { p[3] = 0;		 }
			else		 		   { p[3] = p[3] - temp;  }
		
		}	
	}

	return temp;
}

// ========================== SCALE PIXBUF to 300x300 max ================================= //
// Scale the image to the maxium size of max_x and max_y
GdkPixbuf* interface::scale_pixbuf(GdkPixbuf * pixbuf, int max_x, int max_y)
{
	g_debug("Scaling pixbuf to %ix%i",max_x,max_y);
	
	int x = gdk_pixbuf_get_width  (pixbuf);
	int y = gdk_pixbuf_get_height (pixbuf);

	bool modify = false;
	double value;
	
	if(x > max_x)
	{
		value = max_x/float(x);
		x = x*value;
		y = y*value;
		modify=true;
	}	
	if(y > max_y)  
	{
		value = max_y/float(y);
		x = x*value;
		y = y*value;
		modify=true;
	}

	if(modify)
	{
		value = double(x)/(gdk_pixbuf_get_width(pixbuf));

		g_debug("Scaling Image (%f):%dx%d", value,
			gdk_pixbuf_get_width(pixbuf),
			gdk_pixbuf_get_height(pixbuf));
		g_debug("--> %ix%i",x,y);
	}

	GdkPixbuf * newPixbuf = gdk_pixbuf_scale_simple (pixbuf,x,y,GDK_INTERP_BILINEAR);
	//g_object_unref (pixbuf);
	newPixbuf = gdk_pixbuf_add_alpha(newPixbuf,false,0,0,0);			//Add Alpha Channel
	return newPixbuf;

}


//========================= LAUNCH DEFAULT APP =============== //
void interface::launch_default_app(void)
{
	string cmd = "gnome-open \"" +  m_filename + "\"" ; 
	g_spawn_command_line_async(cmd.c_str(), NULL);
	gtk_main_quit();
}

// ========= SET FULLSCREEN EVENT ============== //
void interface::set_fullscreen_event()
{
	g_debug("Setting Fullscreen Event");
	m_fullscreen_event = true;
}

void interface::unset_fullscreen_event()
{
	g_debug("UnSetting Fullscreen Event");
	m_fullscreen_event = false;
}


void interface::set_error(GError* error) {
	m_error = error;
}

GError * interface::get_error(void) {
	if(m_error)
		return g_error_copy(m_error);
	else
		return NULL;
}

bool interface::is_program_available( const char * prog) {
	char * str;
	str = g_find_program_in_path (prog);
	if (str != NULL) {
		g_free (str);
		return true;
	}
	return false;
}

gboolean interface::fullscreen_trick(GtkWidget *widget,
	GdkEventButton *event,
	gpointer prInstance)
{
	//TODO: Call the singleton fullscreen, not the iVideo one...
	interface * instance = (interface*)prInstance;
	g_debug("Launching Fullscreen Signal");
	//instance->set_fullscreen_event();
	instance->m_fullscreen_event = false;
	Gloobus::instance()->fullscreen();
	return false;
};

gboolean interface::defaultapp_trick(GtkWidget *widget,
	GdkEventButton *event,
	gpointer prInstance)
{
	//TODO: Call the singleton fullscreen, not the iVideo one...
	interface * instance = (interface*)prInstance;
	g_debug("Launching Fullscreen Signal");
	instance->launch_default_app();
	return false;						
};
