#ifndef _INTERFACE_
#define _INTERFACE_

#include <map>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include <string>
#include <iostream>
#include <gtk/gtk.h>

#include <stdlib.h>
#include <gio/gio.h>

/*Translations*/
#include <libintl.h>
#include <locale.h>
#define _(String) gettext(String)

class interface;

#include "gloobus-preview-defines.h"
#include "gloobus-preview-singleton.h"
#include "config.h"

using namespace std;

class interface
{
	public:
				interface();
		virtual ~interface();

	public:
		bool			m_fullscreen_event;
	
	protected:		
		string			m_filename;
		string			m_filetype;
		string			m_uri;
		GFile		*	m_gfile;
		GFileInfo	*	m_file_info;
		GtkWidget	*	m_info;
		GError		*	m_error;
	
		//Control Buttons Variables
		GtkWidget	*	m_fullscreen_button;
		GtkWidget	*	m_defaultapp_button;
		GtkWidget	*	m_fullscreen_box;
		GtkWidget	*	m_defaultapp_box;
		GtkWidget	*	m_hbox;

	public:
	// These function need to be implemented by plugins or its interface!!
		virtual int		get_width			( void ) = 0;
		virtual int		get_height			( void ) = 0;
		virtual bool	load				( void ) = 0;
		virtual void	draw				( GtkContainer * container ) = 0;
		virtual void	draw_toolbar		( GtkWidget * container ) = 0;
		virtual void	end					( void ) = 0;
		virtual bool	can_fullscreen		( void ) = 0;
		virtual void	draw_fullscreen		( GtkContainer * container ) = 0;
		virtual void	draw_unfullscreen	( GtkContainer * container ) = 0;
		virtual void	key_press_event		(int) {};
		
				bool	set_filename	 		( string filename );
				void	set_file_info	 		( void );
				string	get_file_size_string	( string filename );
				double	get_file_size	 		( string filename );
				string	get_file_info_string 	( void );
				string	get_safe_type	 		( bool replace_slash );				//Mime Type
				int		get_screen_width 		( void );
				int		get_screen_height		( void );
				void	trash_file				( void );
	
				void	draw_file_info			( GtkContainer * container );
				void	show_info				( void );
				void	hide_info				( void );
				void 	draw_properties			( GtkContainer * container , string text , int x, int y);
				void	draw_controls			( GtkContainer * container );
				void	draw_default_toolbar	( GtkWidget * container );
				void	show_fullscreen_controls	(GtkContainer * container);
				void	show_unfullscreen_controls	(GtkContainer * container);
				void	set_fullscreen_event	( void );
				void	unset_fullscreen_event	( void );
				void	launch_default_app		( void );

				GError*	get_error				( void );
				void	set_error				( GError* );

				//Pixbuf Opreations
				GdkPixbuf*	get_icon				( void );
				GdkPixbuf*	pixbuf_reflection		( GdkPixbuf *pixbuf);
				GdkPixbuf*	scale_pixbuf			( GdkPixbuf *pixbuf, int max_x = 128, int max_y=128 );


	static gboolean fullscreen_trick(GtkWidget*, GdkEventButton*, gpointer );
	static gboolean defaultapp_trick(GtkWidget*, GdkEventButton*, gpointer );
	static bool is_program_available( const char * );
};

// ================================================================================ //
struct plugin_filetype
{
		string name;			//Plugin Name
		string filetype;		//Filetype that plugins reads
		string type;			//If image, document, music, video... It will be declared in the interface
		int priority;			//Priority of use this plugin before another
};

typedef int getVersion_t();									//GET VERSION FUNCTION, ALL PLUGINS MUST HAVE IT
extern vector<plugin_filetype> pluginVector;				//VECTOR TO STORE WICH FILETYPE READ EACH PLUGIN

#endif
