#include "gloobus-preview-interface-music.h"

// =========================== ~iMUSIC ======================================== //
iMusic::~iMusic()
{
	g_debug("Destroying iMusic");
	g_source_remove(m_timeout_id);
    g_object_unref(m_pixbuf);
    g_object_unref(m_reflection);
}

// =============================== LOAD ======================================= //
bool iMusic::load			( void )
{
	g_debug("Loading iMusic");
    GdkPixbuf* temp = get_pixbuf();
	m_pixbuf = scale_pixbuf(temp,ICON_SIZE,ICON_SIZE);		//Scales the cover to normal 200x200 max
	m_reflection = pixbuf_reflection(m_pixbuf);						//Creates the reflection Pixbuf
    g_object_unref(temp);
	m_playing = true;

	return true;
}
// =============================== END ======================================= //
void iMusic::end			( void )
{
	g_debug("Ending iMusic");
	stop();
}

int	iMusic::get_width		( void ){return 450;}
int iMusic::get_height		( void ){return 280;}

// ================================== DRAW  =================================== //
void iMusic::draw (GtkContainer * container)
{
	draw_cover		( GTK_CONTAINER(container ));
	draw_file_info	( GTK_CONTAINER(container ));
	draw_controls	( GTK_CONTAINER(container ));
	play();
	m_timeout_id = g_timeout_add (50, &iMusic::trick2, this);				//Get new position
}

// =========================== DRAW IMAGE ================================= //
void iMusic::draw_cover(GtkContainer * container)
{

	GtkWidget *image 				= gtk_image_new_from_pixbuf (m_pixbuf);
	GtkWidget *image_container		= gtk_fixed_new ();
	GtkWidget *reflection  			= gtk_image_new_from_pixbuf (m_reflection);
	GtkWidget *reflection_container	= gtk_fixed_new ();

	int x = SHADOW_WIDTH;			//Default values are at top left
	int y = HEADER_HEIGHT + SHADOW_WIDTH;			//and 20 from top (For the header)

	if(gdk_pixbuf_get_width(m_pixbuf) < 300)		//Minum width
	{
		x = (150 + SHADOW_WIDTH ) - gdk_pixbuf_get_width(m_pixbuf)/2;
	}
	if(gdk_pixbuf_get_height(m_pixbuf) < 300)		//Minium height
	{
		y = (150 + SHADOW_WIDTH + HEADER_HEIGHT) - gdk_pixbuf_get_height(m_pixbuf)/2;
	}


	gtk_fixed_put 	  	(GTK_FIXED(image_container),image,25+SHADOW_WIDTH, 50 + SHADOW_WIDTH + HEADER_HEIGHT);
	gtk_fixed_put 	  	(GTK_FIXED(reflection_container),reflection,25+SHADOW_WIDTH, 50 + SHADOW_WIDTH + HEADER_HEIGHT + gdk_pixbuf_get_height(m_pixbuf));
	gtk_container_add 	(GTK_CONTAINER(container),image_container);
	gtk_container_add 	(GTK_CONTAINER(container),reflection_container);
	gtk_widget_show   	(image_container);
	gtk_widget_show   	(image);
	gtk_widget_show	 	(reflection_container);
	gtk_widget_show	 	(reflection);

};


// ================================== DRAW FILE INFO =================================== //
void iMusic::draw_file_info(GtkContainer * container)
{
	string text = get_song_name() + "\n\n" + get_song_artist() + "\n\n" + get_song_album();
	draw_properties ( container, text, 200 , 75 + SHADOW_WIDTH + HEADER_HEIGHT);

}
// ================================== DRAW CONTROLS =================================== //
void iMusic::draw_controls(GtkContainer * container)
{
		GdkColor color;
		gdk_color_parse ("white", &color);

		string buttonstr		= (string)PACKAGE_DATA_PATH + "/media-pause.svg";
		
		m_play_pause_button		= gtk_image_new_from_file(buttonstr.c_str());
		m_event_box				= gtk_event_box_new();
								  gtk_event_box_set_visible_window(GTK_EVENT_BOX(m_event_box),false);
								  gtk_container_add(GTK_CONTAINER(m_event_box),m_play_pause_button);

		m_duration_label 		= gtk_label_new("--:--");
		m_current_time_label 	= gtk_label_new("00:00");

		//Set white color to labels
		gtk_widget_modify_fg (m_duration_label, GTK_STATE_NORMAL, &color);
		gtk_widget_modify_fg (m_current_time_label, GTK_STATE_NORMAL, &color);

		//m_adjustment = gtk_adjustment_new (0, 0, 101, 1, 5, 25);
       	//m_scale = gtk_hscale_new (GTK_ADJUSTMENT (m_adjustment));
		m_scale = gtk_hscale_new_with_range		(0, 100, 1);
		gtk_scale_set_draw_value				(GTK_SCALE(m_scale), false);			//Do not draw the numbers

		GtkWidget * vbox = gtk_vbox_new   	(false,3);
		GtkWidget * hbox = gtk_hbox_new   	(false,3);
		GtkWidget * hbox2 = gtk_hbox_new   	(false,3);


		gtk_box_pack_end 	(GTK_BOX (hbox), m_event_box		, 		true, 	false, 	2);
		gtk_box_pack_start 	(GTK_BOX (hbox2), m_current_time_label, 	false, 	false, 	2);
		gtk_box_pack_start 	(GTK_BOX (hbox2), m_scale, 	 				true, 	true, 	2);
       	gtk_box_pack_end 	(GTK_BOX (hbox2), m_duration_label, 	 	false, 	false, 	2);
		gtk_box_pack_start	(GTK_BOX (vbox), hbox,						false,	false,	2);
		gtk_box_pack_end	(GTK_BOX (vbox), hbox2, 					true,	true,	2);
		
		
	
		gtk_fixed_put			( GTK_FIXED(container),vbox, 25 , get_height() -5 );
	    gtk_widget_set_usize	( hbox, get_width()-20,  35);			
		gtk_widget_show_all		( vbox );

		//When the seek bar is moved, change the position
		g_signal_connect (G_OBJECT (m_scale), 		"change-value", 		G_CALLBACK (&iMusic::trick), 			this);
		g_signal_connect (G_OBJECT (m_event_box), 	"button-press-event",   G_CALLBACK (&iMusic::play_pause_trick), this);
}

// ================================== DRAW POSITION =================================== //
// Request the postion to the plugin and puts the range to this new position
void iMusic::draw_position( void )
{
	//g_debug("POSITION => %f\n", get_position());
	gtk_range_set_value (GTK_RANGE (m_scale), get_position());
	//g_debug("DURATION => %s\n", get_duration().c_str());
	gtk_label_set_text (GTK_LABEL(m_duration_label)		,get_duration().c_str()		);
	gtk_label_set_text (GTK_LABEL(m_current_time_label)	,get_current_time().c_str()	);
}


//============================== PLAY PAUSE ========================= //
void iMusic::play_pause( void )
{
	g_debug("Play Pause clicked");
	//gtk_button_set_label (GTK_BUTTON(m_play_pause_button)	,"JOJO"	);
		
	if(m_playing)
	{ 
		pause();
		string buttonstr		= (string)PACKAGE_DATA_PATH + "/media-play.svg";
		gtk_image_set_from_file(GTK_IMAGE(m_play_pause_button),buttonstr.c_str() );
		m_playing = false;
	}
	else		
	{ 
		play();
		string buttonstr		= (string)PACKAGE_DATA_PATH + "/media-pause.svg";
		gtk_image_set_from_file(GTK_IMAGE(m_play_pause_button),buttonstr.c_str() );
		m_playing = true; 
	}

}

// ====================== FULLSCREEN ================= //
bool iMusic::can_fullscreen  ( void ){ return false;}						//Returns true if the plugin can do fullscreen
void iMusic::draw_fullscreen ( GtkContainer * container )
{
}
void iMusic::draw_unfullscreen ( GtkContainer * container )
{
}

// ============================ DRAW FLOATING CONTROLS ================================ //
void iMusic::draw_toolbar	( GtkWidget * container )
{
    g_debug("iMusic toolbar buttons\n");

	draw_default_toolbar(container);
}

