#include "gloobus-preview-interface-video.h"

// @param obj This is the pointer to the class that will call the function load2
//
// =========================== LAUNCH MEMBER FUNCTION ======================================== //
// Trick to launch a the play function, after the widget is created <g_idle_add> 
static gboolean LaunchMemberFunction(void *obj)
{
    iMovie *myObj = reinterpret_cast<iMovie *>(obj);
    myObj->play();
    myObj->create_timeout();
    return FALSE;
};


// =========================== ~iMOVIE ======================================== //
iMovie::~iMovie()
{
	g_debug("Destroying iMovie");
	g_source_remove(m_timeout_id);
}
//============================= END =========================================== //
void iMovie::end()
{
	g_debug("Ending iMovie");
	stop();
}

bool iMovie::load(){
	g_debug("Loading iMovie");
	m_playing = true;

	return true;
}
int  iMovie::get_width	() 	{ return 725;}   
int  iMovie::get_height	() 	{ return 405+35;}	//SEEK BAR SPACE

// ======================= DRAW ================================== //
void iMovie::draw ( GtkContainer * container )
{
	draw_movie		( GTK_CONTAINER(container) );
	draw_controls	( GTK_CONTAINER(container) );

}

// ======================= DRAW MOVIE ================================== //
void iMovie::draw_movie(GtkContainer * container)
{
	GdkColormap *colormap;   			// in this Main Function
	GdkScreen 	*screen;
	screen 	 = gdk_screen_get_default 	   (); 
	colormap = gdk_screen_get_rgb_colormap(screen);
	gtk_widget_set_default_colormap	( colormap ); 
	
	gtk_widget_push_colormap(colormap);
	m_container = GTK_WIDGET(container);
	
	m_drawable = gtk_drawing_area_new();
	gtk_widget_set_size_request (m_drawable, get_width(), get_height() - (SHADOW_WIDTH * 2) - HEADER_HEIGHT);

	gtk_widget_show(m_drawable);
	gtk_fixed_put(GTK_FIXED(m_container),m_drawable,SHADOW_WIDTH,SHADOW_WIDTH+HEADER_HEIGHT);
	gtk_widget_show (m_drawable);

	//Play when the widgets are created
	g_idle_add(LaunchMemberFunction,this);		//Wait till the windows are created and displayed before
												//Do the movie sync									
	gtk_widget_pop_colormap();
	
	//draw_properties ( container, get_file_info_string(), 25 + SHADOW_WIDTH, get_height() + SHADOW_WIDTH + HEADER_HEIGHT - 5); //35 = seek bar
}

// ================================== DRAW CONTROLS =================================== //
void iMovie::draw_controls(GtkContainer * container)
{
		GdkColor color;
		gdk_color_parse ("white", &color);

		string playbuttonstr	= (string)PACKAGE_DATA_PATH + "/media-pause.svg";
		
		m_play_pause_button		= gtk_image_new_from_file(playbuttonstr.c_str());
		m_playevent_box			= gtk_event_box_new();
								  gtk_event_box_set_visible_window(GTK_EVENT_BOX(m_playevent_box),false);
								  gtk_container_add(GTK_CONTAINER(m_playevent_box),m_play_pause_button);
								  
		string fullbuttonstr	= (string)PACKAGE_DATA_PATH + "/view-fullscreen.svg";
								  
		m_fullscreen_button		= gtk_image_new_from_file(fullbuttonstr.c_str());
		m_fullevent_box			= gtk_event_box_new();
								  gtk_event_box_set_visible_window(GTK_EVENT_BOX(m_fullevent_box),false);
								  gtk_container_add(GTK_CONTAINER(m_fullevent_box),m_fullscreen_button);

		m_duration_label 		= gtk_label_new("--:--");
		m_current_time_label 	= gtk_label_new("00:00");

		gtk_widget_modify_fg (m_duration_label, GTK_STATE_NORMAL, &color);
		gtk_widget_modify_fg (m_current_time_label, GTK_STATE_NORMAL, &color);

		m_scale = gtk_hscale_new_with_range		(0, 100, 1);
		gtk_scale_set_draw_value				(GTK_SCALE(m_scale), false);			//Do not draw the numbers

					m_vbox 	= gtk_vbox_new   	(false,3);
		GtkWidget * hbox 	= gtk_hbox_new   	(false,3);		//Buttons
		GtkWidget * hbox2 	= gtk_hbox_new   	(false,3);		//Durations

		gtk_box_pack_start 		( GTK_BOX (hbox),   gtk_label_new(""), 		true, 	true, 	2);
		gtk_box_pack_start 		( GTK_BOX (hbox),   m_playevent_box	, 		false, 	false, 	2);
		gtk_box_pack_start 		( GTK_BOX (hbox),   m_fullevent_box	, 		false, 	false, 	2);
		gtk_box_pack_start 		( GTK_BOX (hbox),   gtk_label_new(""), 		true, 	true, 	2);


		gtk_box_pack_start 		( GTK_BOX (hbox2),  m_current_time_label, 	false, 	false, 	2);
		gtk_box_pack_start 		( GTK_BOX (hbox2),  m_scale, 	 			true, 	true, 	2);
       	gtk_box_pack_end 		( GTK_BOX (hbox2),  m_duration_label, 	 	false, 	false, 	2);

		gtk_box_pack_start		( GTK_BOX (m_vbox), hbox,					false,	false,	2);
		gtk_box_pack_end		( GTK_BOX (m_vbox), hbox2, 					true,	true,	2);
	
		gtk_fixed_put			( GTK_FIXED(container), m_vbox, 25 , get_height() -5 );
	    gtk_widget_set_usize	( hbox, get_width()-20,  35);			
		gtk_widget_show_all		( m_vbox );
		

		//When the seek bar is moved, change the position
		g_signal_connect (G_OBJECT (m_scale), 			"change-value", 		G_CALLBACK (&iMovie::trick), this);
		g_signal_connect (G_OBJECT (m_playevent_box), 	"button-press-event",   G_CALLBACK (&iMovie::play_pause_trick), this);
		g_signal_connect (G_OBJECT (m_fullevent_box), 	"button-press-event",   G_CALLBACK (&iMovie::fullscreen_trick), this);
}

// ============================ DRAW FLOATING CONTROLS ================================ //
void iMovie::draw_toolbar	( GtkWidget * container )
{
    g_debug("iMovie toolbar buttons\n");

	//Fullscreen button
	string fullbuttonstr			= 	(string)PACKAGE_DATA_PATH + "/view-restore.svg";
	GtkWidget* fullscreen_button	= 	gtk_image_new_from_file(fullbuttonstr.c_str());
	GtkWidget* fullscreen_box		= 	gtk_event_box_new();
							  			gtk_event_box_set_visible_window(GTK_EVENT_BOX(fullscreen_box),false);
							  			gtk_container_add(GTK_CONTAINER(fullscreen_box),fullscreen_button);
	
	//Launch Default App buton
	string defaultappstr			= 	(string)PACKAGE_DATA_PATH + "/media-pause.svg";
	m_play_pause_button_toolbar	    = 	gtk_image_new_from_file(defaultappstr.c_str());
	GtkWidget* defaultapp_box		= 	gtk_event_box_new();
							  			gtk_event_box_set_visible_window(GTK_EVENT_BOX(defaultapp_box),false);
							  			gtk_container_add(GTK_CONTAINER(defaultapp_box),m_play_pause_button_toolbar);

	GtkWidget *vbox		= gtk_vbox_new	  (false, 3);
	GtkWidget *hbox 	= gtk_hbox_new    (false, 3);
	
	gtk_box_pack_start 		( GTK_BOX (hbox),   gtk_label_new(""), 		true, 	true, 	2);
	gtk_box_pack_start 		( GTK_BOX (hbox),   defaultapp_box	, 		false, 	false, 	2);
	gtk_box_pack_start 		( GTK_BOX (hbox),   fullscreen_box	, 		false, 	false, 	2);
	gtk_box_pack_start 		( GTK_BOX (hbox),   gtk_label_new(""), 		true, 	true, 	2);	

	gtk_box_pack_start 		( GTK_BOX (vbox),   gtk_label_new(""), 		false, 	false, 	5);
	gtk_box_pack_start 		( GTK_BOX (vbox),   hbox, 					false, 	false, 	5);	


	g_signal_connect 	( G_OBJECT (fullscreen_box), 	"button-press-event",   G_CALLBACK (&iMovie::fullscreen_trick), this);
	g_signal_connect 	( G_OBJECT (defaultapp_box), 	"button-press-event",   G_CALLBACK (&iMovie::play_pause_trick), this);

	//gtk_box_pack_start ( GTK_BOX(container), hbox, false,false,2);
	gtk_container_add ( GTK_CONTAINER(container), vbox);

	gtk_widget_show_all(container);
   
}

// ================================== HIDE CONTROLS =================================== //
void iMovie::hide_controls		( void )
{
	gtk_widget_hide(m_vbox);
}
// ================================== SHOW CONTROLS =================================== //
void iMovie::show_controls 		( void )
{
	gtk_widget_show_all(m_vbox);
}

// ================================== DRAW POSITION =================================== //
// Request the postion to the plugin and puts the range to this new position
void iMovie::draw_position( void )
{
	gtk_range_set_value	(GTK_RANGE	(m_scale)				,get_position()				);
	gtk_label_set_text	(GTK_LABEL	(m_duration_label)		,get_duration().c_str()		);
	gtk_label_set_text	(GTK_LABEL	(m_current_time_label)	,get_current_time().c_str()	);
}

// =========================== CREATE TIMEOUT ================================== //
void iMovie::create_timeout()
{
	 m_timeout_id = g_timeout_add 	(100, &iMovie::trick2,this);				//Get new position
}
//============================== PLAY PAUSE ========================= //
void iMovie::play_pause( void )
{
	g_debug("Play Pause clicked");
		
	if(m_playing)
	{ 
		pause();
		string buttonstr		= (string)PACKAGE_DATA_PATH + "/media-play.svg";
		gtk_image_set_from_file(GTK_IMAGE(m_play_pause_button),         buttonstr.c_str() );
		gtk_image_set_from_file(GTK_IMAGE(m_play_pause_button_toolbar), buttonstr.c_str() );
		m_playing = false;
	}
	else		
	{ 
		play();
		string buttonstr		= (string)PACKAGE_DATA_PATH + "/media-pause.svg";
		gtk_image_set_from_file(GTK_IMAGE(m_play_pause_button),         buttonstr.c_str() );
		gtk_image_set_from_file(GTK_IMAGE(m_play_pause_button_toolbar), buttonstr.c_str() );
		m_playing = true; 
	}

}
// ====================== FULLSCREEN ================= //
bool iMovie::can_fullscreen  ( void ){ return true;}						//Returns true if the plugin can do fullscreen
void iMovie::draw_fullscreen ( GtkContainer * container )
{
	gtk_widget_set_size_request (m_drawable, get_screen_width(), get_screen_height());
	gtk_fixed_move  (GTK_FIXED(m_container),  m_drawable,	0,	0);
	gtk_fixed_move	(GTK_FIXED(m_container),  m_vbox, 	get_screen_width()/2 - get_width()/2, 			get_screen_height() -85 );

	GdkColor black; black.red = 0; black.green=0; black.blue=0;
	gtk_widget_modify_bg (m_drawable, GTK_STATE_NORMAL, &black);
	hide_controls();
}
// ====================== UNFULLSCREEN ================= //
void iMovie::draw_unfullscreen ( GtkContainer * container )
{
	gtk_widget_set_size_request (m_drawable, get_width(), get_height() - (SHADOW_WIDTH * 2) - HEADER_HEIGHT);
	gtk_fixed_move  (GTK_FIXED(m_container),m_drawable,	SHADOW_WIDTH,SHADOW_WIDTH+HEADER_HEIGHT);
	gtk_fixed_move	(GTK_FIXED(m_container),  m_vbox, 	25 , get_height() -5 );
	show_controls();
}
