//Create the POT file
//xgettext -d gloobus-preview --keyword=_ -s -o gloobus-preview.pot gloobus-preview*.cpp

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*Translations*/
#include <libintl.h>
#include <locale.h>
#define _(String) gettext(String)

#include <cstdlib>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
//#include <cairo.h>
#include <string>

#include "gloobus-preview-interface-main.h"
//#include "iImage.h"
//#include "iText.h"
//#include "iMusic.h"
//#include "iDocument.h"
//#include "iMovie.h"
//#include "iList.h"

#include "gloobus-preview-config.h"
#include "gloobus-preview-window.h"
#include "gloobus-preview-defines.h"
#include "gloobus-preview-singleton.h"

#include "gloobus-preview-interface-main.h"
#include "gloobus-preview-interface-image.h"
#include "gloobus-preview-interface-text.h"
#include "gloobus-preview-interface-music.h"
#include "gloobus-preview-interface-document.h"
#include "gloobus-preview-interface-video.h"
#include "gloobus-preview-interface-list.h"
#include "gloobus-preview-interface-free.h"
#include "gloobus-preview-interface-default.h"

using namespace std;

//GLOBAL VARIABLES TO STORE PLUGINS INFORMATION
vector<plugin_filetype> pluginVector;				          		  	//VECTOR TO STORE WICH FILETYPE READ EACH PLUGIN

map <string, create_t_image  *, less<string> > factory_image;		  	//Global variable that plugin uses to register
map <string, destroy_t_image *, less<string> > factoryDestroy_image;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_music  *, less<string> > factory_music;		  	//Global variable that plugin uses to register
map <string, destroy_t_music *, less<string> > factoryDestroy_music;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_movie  *, less<string> > factory_movie;		  	//Global variable that plugin uses to register
map <string, destroy_t_movie *, less<string> > factoryDestroy_movie;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_text   *, less<string> > factory_text;		  	//Global variable that plugin uses to register
map <string, destroy_t_text  *, less<string> > factoryDestroy_text;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_document   *, less<string> > factory_document;		  	//Global variable that plugin uses to register
map <string, destroy_t_document  *, less<string> > factoryDestroy_document;   //MAP WITH NAME AND DESTRUCTOR

map <string, create_t_list   *, less<string> > factory_list;		  	//Global variable that plugin uses to register
map <string, destroy_t_list  *, less<string> > factoryDestroy_list;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_free   *, less<string> > factory_free;		  	//Global variable that plugin uses to register
map <string, destroy_t_free  *, less<string> > factoryDestroy_free;   	//MAP WITH NAME AND DESTRUCTOR


void destroy   	( GtkWidget *widget, gpointer data );
void log_handler(const gchar *log_domain, GLogLevelFlags log_level,const gchar *message, gpointer user_data);


// ======================================= MAIN ===================================== //
int main (int argc, char *argv[])
{
	//g_thread_init(NULL);
	//gdk_threads_init();

	//GETTEXT Initialization
    setlocale( LC_ALL, "" );
    bindtextdomain	( "gloobus-preview", LOCALEDIR);
    textdomain		( "gloobus-preview" );

	// GTK INITIALIZATION	
	gtk_init (&argc, &argv);

	//Load Custom RC
	//string gtkstr = (string)PACKAGE_DATA_PATH + "/gtkrc/customgtk.rc";
	//string pixmap_source = "pixmap_path = \"" + (string)PACKAGE_DATA_PATH + "/gtkrc\"";
	//gtk_rc_parse			(gtkstr.c_str());
    //gtk_rc_parse_string     (pixmap_source.c_str());
    gtk_rc_parse(PACKAGE_DATA_PATH "/gtkrc/customgtk.rc");

    gloobusConfig config;
    config.parse_args(argc, argv);

	bool debug = config.get_cli_debug();

	g_log_set_handler (NULL, (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                     | G_LOG_FLAG_RECURSION), log_handler, (void*)debug);

	if(debug) g_message(_("Debug mode: On"));

	switch(config.get_cli_action()) {
		case GP_ACTION_PREVIEW:
			Gloobus::instance()->show_preview( config.get_cli_target() );
			break;
		case GP_ACTION_FILETYPES:
			Gloobus::instance()->show_supported_filetypes();
			break;
		case GP_ACTION_HELP:
			Gloobus::instance()->show_help();
			break;
	}

	return 0;
}

// ===================================== DESTROY ===================================== //
// This function is called when a exit event is recived
void destroy ( GtkWidget *widget, gpointer data )
{
    gtk_main_quit ();
}

// =================================== LOG_HANDLER =================================== //
void log_handler  (const gchar *log_domain,
	GLogLevelFlags log_level,
	const gchar *message,
	gpointer debug)
{
	if(!(bool)debug && G_LOG_LEVEL_DEBUG == log_level) {
		/*do nothing */
	} else {
		char add[50]="";
		if (log_level==  G_LOG_LEVEL_MESSAGE) {
			strcpy(add, CLI_COLOR_BOLD    "[INFO]"     CLI_COLOR_RESET);
		} else  if (log_level == G_LOG_LEVEL_DEBUG)   {
			strcpy(add, CLI_COLOR_BLUE    "[DEBUG]"    CLI_COLOR_RESET);
		} else  if (log_level==  G_LOG_LEVEL_WARNING) {
			strcpy(add, CLI_COLOR_RED     "[WARNING]"  CLI_COLOR_RESET);
		} else if (log_level==  G_LOG_LEVEL_CRITICAL) {
			strcpy(add, CLI_COLOR_DARKRED "[CRITICAL]" CLI_COLOR_RESET);
		}
		g_print("%s %s\n", add, message);
	}
}


