#include "gloobus-preview-singleton.h"

#include "gloobus-preview-interface-main.h"
#include "gloobus-preview-interface-image.h"
#include "gloobus-preview-interface-text.h"
#include "gloobus-preview-interface-music.h"
#include "gloobus-preview-interface-document.h"
#include "gloobus-preview-interface-video.h"
#include "gloobus-preview-interface-list.h"
#include "gloobus-preview-interface-free.h"
#include "gloobus-preview-interface-default.h"

//// ============================= SINGLETON =========================== //
Gloobus * Gloobus::m_instance = 0;
Gloobus * Gloobus::instance()
{ 
  	if (m_instance == NULL)
  	{
  		m_instance = new Gloobus();
  	}
  	return m_instance; 
}

Gloobus::Gloobus()
	:m_PManager(new pluginManager() )
	,m_gwindow(0)
	,m_fullscreen(false)
	,m_gloobus_interface(0)
	,m_timeoutID(0)
	,m_error(0)
{
	g_debug("Creating Gloobus class");
}

//// ============================== INIT ================================ //
void Gloobus::init(string filename ,gloobusWin *win)
{
	m_filename 		= filename;
	m_gwindow		= win;
	
	m_PManager->load_plugins	();			//Plugin manager loads the .so

	GdkScreen *screen = gdk_screen_get_default ();
	m_gwindow->set_screen_size(gdk_screen_get_width(screen),
		gdk_screen_get_height(screen));
}
// =============================== UNLOAD PLUGIN ======================= //
// Unload all the plugins
void Gloobus::end()
{
	m_PManager->unload_plugins();						// Unload Plugins
}

void Gloobus::show_preview(char * target)
{
	string filename;
	gloobusWin * win = new gloobusWin();
	win->create_window( DEFAULT_WIDTH, DEFAULT_HEIGHT );
	
	//Events Handlers
	g_signal_connect (win->get_window_pointer(), "key-press-event",
		G_CALLBACK(key_press), NULL);

	win->create_toolbar();
	g_signal_connect (win->get_toolbar_pointer(),"key-press-event",
					  G_CALLBACK(key_press), NULL);
		
	if(target) {
		filename = target;

		// check relative path:
		if (filename.c_str()[0] != '/') {
			filename = "./" + filename;
		}
		
		// strip final slash:
		if (filename.c_str()[filename.size()-1] == '/') {
			filename = filename.substr(0,filename.size()-1);
		}
	} else {
		g_debug("Getting file from clipboard");
		GtkClipboard *clip	= gtk_clipboard_get ( GDK_SELECTION_CLIPBOARD );
		gchar **uri 		= gtk_clipboard_wait_for_uris ( clip );
		if(uri == NULL) {\
			m_error = g_error_new(g_quark_from_string ("Gloobus"), 1,
				_("There is no file in clipboard"));
		} else {
			filename = g_filename_from_uri(uri[0], NULL, NULL);
		}
	}
	init( filename , win );
	load_folder_filenames();
	if( !m_error ) {
		g_debug("Filename: %s", filename.c_str());
		if( !search_actual_filename()) {
			m_error = g_error_new(g_quark_from_string ("Gloobus"), 2,
				_("File not found"));
		} else if(create_plugin()) {

			int w = get_width () + SHADOW_WIDTH * 2;
			int h = get_height() + SHADOW_WIDTH * 2 + BOTTOM_HEIGHT + HEADER_HEIGHT;

			win->set_size( w, h );
			win->set_title( filename );
			win->add_common_buttons();
			win->show_all();

			draw_plugin ();	//Its drawn on top so buttons in fullscreen are hidden
		} else {
			m_error = m_gloobus_interface->get_error();
		}
	}
	if(m_error)
		show_error();
  
    gtk_main ();

	destroy_plugin();//Destroy current plugin
	end();			 //Unload All plugins before leave
}

void Gloobus::show_supported_filetypes  ( )
{
	m_PManager->load_plugins	();			//Plugin manager loads the .so
	m_PManager->show_supported_filetypes();
	m_PManager->unload_plugins();		// Unload Plugins
}

void Gloobus::show_help  ( )
{
	printf(
		"%s:\n"
		"   gloobus-preview [%s...] [URI]\n\n",
		_("Usage"), _("OPTION") );

	printf(
		"   %s\n\n", _(
		"If URI is not provided Gloobus-Preview checks clipboard "
		"for filename"));

	printf("%s:\n", _("Help options") );
	printf("   -h, --help                   %s\n",
		_("Show help options") );
	
	printf("%s:\n", _("Application options") );
	printf("   -d, --debug                  %s\n",
		_("Show debug messages") );
	printf("   --show-supported-filetypes   %s\n",
		_("Show supported filetypes") );

	printf("\n");
}

void Gloobus::show_error( bool idefault ) {
	//g_warning(m_error->message);
	if(idefault || m_gloobus_interface ) {
		g_debug("Loading iDefault");
		
		m_class = "default";
		m_gloobus_interface = new iDefault();
		m_gloobus_interface->set_filename(m_filename);
		m_gloobus_interface->load();

		m_gwindow->set_size(get_width() +  SHADOW_WIDTH*2, get_height()+ SHADOW_WIDTH*2 + BOTTOM_HEIGHT+HEADER_HEIGHT);	
		m_gwindow->set_title		  (m_filename);
		m_gwindow->add_common_buttons ();
		m_gwindow->show_all			  ();
	
		draw_plugin();
	} else {
		g_debug("Loading error dialog");
		gtk_widget_destroy	(GTK_WIDGET(m_gwindow->get_container_pointer()));
		gtk_widget_destroy 	(GTK_WIDGET(m_gwindow->get_toolbar_pointer()));

		m_gwindow->create_toolbar();
		m_gwindow->create_container();
		
		m_gwindow->set_size(
			384 + SHADOW_WIDTH * 2,
			128 + SHADOW_WIDTH * 2 /*+ BOTTOM_HEIGHT*/ + HEADER_HEIGHT);
		m_gwindow->set_title(m_filename);
		m_gwindow->add_common_buttons();
		//GtkWidget * box = gtk_vbox_new(false, 0);
		GtkWidget * icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_ERROR,
			GTK_ICON_SIZE_DIALOG);
		GtkWidget * label = gtk_label_new(m_error->message);
		
		PangoAttrList *info_attrs = pango_attr_list_new();
		pango_attr_list_insert(info_attrs,
			pango_attr_foreground_new(53889,14927,253));
		pango_attr_list_insert(info_attrs,
			pango_attr_weight_new(PANGO_WEIGHT_BOLD));
		pango_attr_list_insert(info_attrs, pango_attr_size_new(14*PANGO_SCALE));
		gtk_widget_set_size_request(label, 256, 128);
		gtk_label_set_attributes(GTK_LABEL(label), info_attrs);
		gtk_label_set_line_wrap(GTK_LABEL(label), true);
		gtk_misc_set_alignment(GTK_MISC(label), 0 , 0.5);
		
		// GTK_ICON_SIZE_DIALOG is 40, for all themes?
		// 128-40 /2 = 44 - place icon in the middle
		gtk_fixed_put(GTK_FIXED(m_gwindow->get_container_pointer()),
			icon, SHADOW_WIDTH+44, HEADER_HEIGHT + SHADOW_WIDTH + 44);
		gtk_fixed_put(GTK_FIXED(m_gwindow->get_container_pointer()),
			label, SHADOW_WIDTH+128, HEADER_HEIGHT + SHADOW_WIDTH);
		gtk_widget_show_all(icon);
		gtk_widget_show_all(label);
		m_gwindow->show_all();
	}
}

// =============================== CREATE PLUGIN ======================= //
// This functions searchs the plugin associated with the <m_filename> and then creates it.
// It also defines the variable <m_gloobus_interface> to be a polymorphism so the default functions
// can be called with directly on this variable
bool Gloobus::create_plugin( )
{
	m_plugin_name = "";
	m_class		  = "";
	
	m_plugin_name 	 = m_PManager->search_for_plugin(m_filename);
	m_class 		 = m_PManager->get_classtype ();
	g_debug("Creating plugin class: %s",m_class.c_str());
	
	if(m_class == "image") {
		g_debug("IMAGE (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_image[m_plugin_name.c_str()](); //Polymorphism
	} else if(m_class == "music") {
		g_debug("MUSIC (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_music[m_plugin_name.c_str()](); //Polymorphism
	} else if(m_class == "text") {
		g_debug("TEXT (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_text[m_plugin_name.c_str()](); //Polymorphism
	} else if(m_class == "document") {
		g_debug("DOCUMENT (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_document[m_plugin_name.c_str()](); //Polymorphism
	} else if(m_class == "list") {
		g_debug("LIST (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_list[m_plugin_name.c_str()](); //Polymorphism
	} else if(m_class == "movie") {
		g_debug("MOVIE (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_movie[m_plugin_name.c_str()](); //Polymorphism
	} else if(m_class == "free") {
		g_debug("FREE (%s)",m_plugin_name.c_str());
		m_gloobus_interface = factory_free[m_plugin_name.c_str()](); //Polymorphism
	} else {
		g_debug("DEFAULT");
		m_class = "default";
		m_gloobus_interface = new iDefault(); //Polymorphism
	}

	if(! m_gloobus_interface->set_filename(m_filename))
		return false;
	if(! m_gloobus_interface->load() )
		return false;

	//FIXME: Do it with signals :
	//m_timeoutID = g_timeout_add	(200, &Gloobus::check_fullscreen_trick,this);
	
	return true;
}
// ============================= DESTROY PLUGIN ======================= //
void Gloobus::destroy_plugin()
{
	g_debug("Destroying Plugin:");
	if(m_timeoutID) {
		g_source_remove(m_timeoutID);
		m_timeoutID = 0;
	}
	
	if(m_gloobus_interface) {
		m_gloobus_interface->end(); //Call a ending function before the destructor
									 //If we don't do this, thread functions
									 //calling virtual plugin function may crash
		if(m_class == "image") {
			factoryDestroy_image[m_plugin_name.c_str()](
				(iImage*)m_gloobus_interface);
		} else if(m_class == "music") {
			factoryDestroy_music[m_plugin_name.c_str()](
				(iMusic*)m_gloobus_interface);
		} else if(m_class == "movie") {
			factoryDestroy_movie[m_plugin_name.c_str()](
				(iMovie*)m_gloobus_interface);
		} else if(m_class == "text") {
			factoryDestroy_text[m_plugin_name.c_str()](
				(iText*)m_gloobus_interface);
		} else if(m_class == "document") {
			factoryDestroy_document[m_plugin_name.c_str()](
				(iDocument*)m_gloobus_interface);
		} else if(m_class == "list"){
			factoryDestroy_list[m_plugin_name.c_str()](
				(iList*)m_gloobus_interface);
		} else if(m_class == "free"){
			factoryDestroy_free[m_plugin_name.c_str()](
				(iFree*)m_gloobus_interface);
		} else if(m_class == "default"){
			delete m_gloobus_interface;
		}
		m_gloobus_interface = 0;
	}
}
// ============================= GET WIDTH ======================= //
int Gloobus::get_width(void)
{

	return m_gloobus_interface->get_width();

}
// ============================= GET HEIGHT ======================= //
int Gloobus::get_height(void)
{
	return m_gloobus_interface->get_height();

}
// ============================= DRAW PLUGIN ======================= //
void Gloobus::draw_plugin(void)
{
	m_gloobus_interface->draw(GTK_CONTAINER(m_gwindow->get_container_pointer()));

	m_gloobus_interface->draw_toolbar( gtk_dialog_get_content_area( GTK_DIALOG(m_gwindow->get_toolbar_pointer() ) ) );
}
// ============================= RESIZE WINDOW ======================= //
void Gloobus::resize_window	( int x, int y )
{
	gtk_window_resize(GTK_WINDOW(m_gwindow->get_window_pointer()),x,y);
}
// ============================= FULLSCREEN WINDOW ======================= //
void Gloobus::fullscreen (void)
{
	m_gloobus_interface->unset_fullscreen_event();

	GdkScreen *screen = gtk_window_get_screen(GTK_WINDOW(m_gwindow->get_window_pointer()));
	if(m_gloobus_interface->can_fullscreen())
	{
		
		if(m_fullscreen)	//UNFULLSCREEN
		{
			m_gwindow->set_size						(get_width()+SHADOW_WIDTH*2 ,get_height()+SHADOW_WIDTH*2+HEADER_HEIGHT+BOTTOM_HEIGHT);	//Draw in center
			m_gwindow->show_common_buttons			();		
			m_gwindow->hide_toolbar					();	
			m_gloobus_interface->draw_unfullscreen	(GTK_CONTAINER(m_gwindow->get_container_pointer()));
			gtk_widget_hide_all						(m_gwindow->get_toolbar_pointer());
			gtk_window_unfullscreen					(GTK_WINDOW(m_gwindow->get_window_pointer()));
			m_fullscreen = false;
		}
		else				//FULLSCREEN
		{
			m_gwindow->set_size						(gdk_screen_get_width(screen),gdk_screen_get_height(screen));			//Draw in center
			m_gwindow->hide_common_buttons			();			
			m_gwindow->show_toolbar					();
			//gtk_widget_set_size_request       	(m_gwindow->get_window_pointer(),gdk_screen_get_width(screen),gdk_screen_get_height(screen));
			m_gloobus_interface->draw_fullscreen	(GTK_CONTAINER(m_gwindow->get_container_pointer()));
			gtk_window_fullscreen					(GTK_WINDOW(m_gwindow->get_window_pointer()));
			m_fullscreen = true;
		}

	}
	

}
// ============================= Load filenames ============================== //
void Gloobus::load_folder_filenames  ( void )
{
	string folder;
	
	if(m_filename.empty()) {
		char* dir = g_get_current_dir();
		folder = string(dir);
		g_free(dir);
	} else {
		folder =   m_filename.substr(0,m_filename.rfind("/"));
	}
	
	GFile * directory = g_file_new_for_path(folder.c_str());
	GFileEnumerator * directory_files = g_file_enumerate_children(directory,
		G_FILE_ATTRIBUTE_STANDARD_NAME, G_FILE_QUERY_INFO_NONE, NULL, NULL);
	
	GFileInfo * info;

	g_debug("FOLDER FILES");
	do {
		info = g_file_enumerator_next_file   (directory_files, NULL, NULL);
		if(info) {
			g_debug("    Files: %s",g_file_info_get_name(info));
			m_files.push_back (
				string(folder + "/" + g_file_info_get_name(info)));
			g_object_unref(info);
		}
	}while(info!=NULL);

    g_object_unref(directory);
    g_object_unref(directory_files);

	m_it = m_files.begin();
    std::sort( m_files.begin(), m_files.end() );

}
// ============================= Search actual filename ======================= //
bool Gloobus::search_actual_filename ( void )
{
	for ( m_it=m_files.begin() ; m_it < m_files.end(); m_it++ )
	{
		//g_debug("%s - %s\n", m_files[i].c_str(), m_filename.c_str());
		if(*m_it == m_filename)
		{
			 return true;
		}
	}
	return false;
}
// ============================= Load Next File ============================== //
void Gloobus::next_file ( void )
{
	//m_current_file++;
	//if((unsigned int)m_current_file == m_files.size()) m_current_file = 0;
	if (m_it == m_files.end()-1 or m_it == m_files.end()) 
	{
		g_debug("Volta");
		m_it = m_files.begin();
	}
	else
	{
		m_it++;
	}
	
	m_filename = *m_it;
	g_debug("Loading next file %s", m_filename.c_str());

	end_file_and_new_file();
}
// =============================== Load prev File ============================ //
void Gloobus::prev_file ( void )
{
	//m_current_file--;
	//if(m_current_file < 0) m_current_file = m_files.size() - 1;
	if (m_it == m_files.begin()) 
	{
		g_debug("Volta");
		m_it = m_files.end()-1;
	}
	else
	{
		m_it--;
	}
	
	m_filename = *m_it;
	g_debug("Loading next file %s", m_filename.c_str());
	
	end_file_and_new_file();
}
// =============================== TRASH FILE ================================ //
void Gloobus::trash_file( void )
{
	g_debug("Sending file to trash");
	m_gloobus_interface->trash_file();

	if(m_it == m_files.end()-1)
	{
		m_files.erase(m_it);
		m_it = m_files.begin();
	}
	else
	{
		m_it = m_files.erase(m_it);
	}
	
	m_filename = *m_it;
	end_file_and_new_file();

}
// ============================= END File and new file ======================= //
// This functions clears the current window and prepare it for the new file
// after clearing the window, it draws the new file
void Gloobus::end_file_and_new_file ( void )
{
	destroy_plugin();
	gtk_widget_destroy	(GTK_WIDGET(m_gwindow->get_container_pointer()));
	gtk_widget_destroy 	(GTK_WIDGET(m_gwindow->get_toolbar_pointer()));
	if(m_error) {
		g_error_free(m_error);
		m_error = 0;
	}

	m_gwindow->create_toolbar();
	g_signal_connect (m_gwindow->get_toolbar_pointer(),"key-press-event", 		//And this one, catchs key events
					  G_CALLBACK(key_press), NULL);							//to see when space has being pressed, and quit

	m_gwindow->create_container();							//Create a new container and
	
	if(create_plugin()) {
		m_gwindow->set_size(get_width() +  SHADOW_WIDTH*2, get_height()+ SHADOW_WIDTH*2 + BOTTOM_HEIGHT+HEADER_HEIGHT);	
		m_gwindow->set_title		  (m_filename);
		m_gwindow->add_common_buttons ();
		m_gwindow->show_all			  ();

		draw_plugin();

		if(m_fullscreen)
		{
			if(m_gloobus_interface->can_fullscreen())
			{
				m_fullscreen = false;
				fullscreen();
			}
			else
			{
				gtk_window_unfullscreen		(GTK_WINDOW(m_gwindow->get_window_pointer()));
				//m_fullscreen = false;
			}
		}
	} else {
		m_error = m_gloobus_interface->get_error();
		show_error();
	}
}
// ============================= Launch default application ======================= //
// Launch the gnome-open function for the current file
void Gloobus::launch_default_application ( void )
{
	m_gloobus_interface->launch_default_app();
}



// ===================================== KEY_PRESS =================================== //
// This function is called when there is a key press event
void Gloobus::key_press ( GtkWidget *widget, GdkEventKey *event)
{
    g_message ("Key Pressed %s (%i)",gdk_keyval_name(event->keyval),event->keyval);
	if(event->keyval == 32 || event->keyval == 65307)		//Space or ESC
	{
		gtk_main_quit();
	}
		
	if(event->keyval == 102)				//F key
		Gloobus::instance()->fullscreen();
		
	if(event->keyval == 65363)				//Right Key
		Gloobus::instance()->next_file();

	
	if(event->keyval == 65361)				//Left key
		Gloobus::instance()->prev_file();
		
	if(event->keyval == 65293){			//Enter key
		Gloobus::instance()->launch_default_application();
		gtk_main_quit();
	}

	if(event->keyval == 65535)				//Delete Key
	{
		Gloobus::instance()->trash_file();
	}
	if(Gloobus::instance()->m_gloobus_interface) {
		Gloobus::instance()->m_gloobus_interface->key_press_event(event->keyval);
	}
}

void Gloobus::quit( void ) {
	g_debug("Gloobus is quitting");

	destroy_plugin();

	gtk_main_quit();
}

gboolean Gloobus::check_fullscreen_trick(gpointer prInstance)						
{
	Gloobus * instance = (Gloobus*)prInstance;
	if(instance->m_gloobus_interface->m_fullscreen_event)
	{	
		instance->fullscreen( );		
	}
	return true;
}

void Gloobus::end_file_and_error( void ) {
	if(m_error)
		g_error_free(m_error);
	m_error = m_gloobus_interface->get_error();
	
	destroy_plugin();
	gtk_widget_destroy	(GTK_WIDGET(m_gwindow->get_container_pointer()));
	gtk_widget_destroy 	(GTK_WIDGET(m_gwindow->get_toolbar_pointer()));

	m_gwindow->create_toolbar();
	m_gwindow->create_container();
	g_signal_connect (m_gwindow->get_toolbar_pointer(),"key-press-event", 		//And this one, catchs key events
					  G_CALLBACK(key_press), NULL);

	show_error(1);
}
