//*****************************************************************************************//
// Class that handles all the main plugin classes to provide a unique interrface to 	   //
// be called in mail.cpp																   //
// It will be a singleton so it can be called from callback functions so it can implement  //
// fullscreen and keyboard interactions													   //
//																				     	   //
//*****************************************************************************************//

#ifndef _GLOOBUS_
#define _GLOOBUS_

#include <cstdlib>
#include <dirent.h>
#include <errno.h>
#include <iostream>
#include <algorithm>

class Gloobus;

#include "gloobus-preview-plugin-manager.h"
#include "gloobus-preview-window.h"
#include "gloobus-preview-defines.h"

class Gloobus {
	// =========== Singleton ========= //
	static Gloobus *m_instance;
	Gloobus();
	Gloobus& operator=(Gloobus&);  // Disallowed
	Gloobus(const Gloobus&);       // Disallowed
	
	private:
		pluginManager*	m_PManager;
		string			m_class;
		string			m_plugin_name;
		vector<string>  m_files;
		vector<string>::iterator m_it;

		string			m_filename;
		gloobusWin	*	m_gwindow;
		bool			m_fullscreen;
		interface	*	m_gloobus_interface; //Polymorphism
		int				m_timeoutID;
		GError		*	m_error;

	public:
		static Gloobus* instance        ( ); //Get instance
		void   init                      (string filename, gloobusWin *);	 
		void   show_preview              ( char* );
		void   show_supported_filetypes  ( void );
		void   show_help                 ( void );
		void   show_error                ( bool=0 );
		void   end                       ( void ); //PManager => Unload Plugins
		bool   create_plugin             ( void ); //PManager => Search Plguin and depending on key create one or other
		void   destroy_plugin            ( void ); //Destroy plugin created
		int    get_height                ( void ); //Get plugin height
		int    get_width                 ( void ); //Get plugin width
		void   draw_plugin               ( void );
		void   resize_window             ( int x, int y ); //Resize window
		void   fullscreen                ( void ); //Fullscreen

		void   load_folder_filenames     ( void );
		bool   search_actual_filename    ( void );
		void   next_file                 ( void ); //Unloads current and load next
		void   prev_file                 ( void ); //Unloads current and load prev
		void   trash_file                ( void );
		void   end_file_and_new_file     ( void ); //Destroys the container and all its containers
		void   end_file_and_error        ( void );
	
		void   launch_default_application( void ); //Gnome-open <filename> 

		static gboolean check_fullscreen_trick(gpointer prInstance);
		static void key_press ( GtkWidget *widget, GdkEventKey *event);

		void quit( void );
};

#endif
