#include "gloobus-preview-window.h"

/*Translations*/
#include <libintl.h>
#include <locale.h>
#define _(String) gettext(String)

gloobusWin::gloobusWin()
	:m_toolbar_bg(0)
{}

// @param w window width
// @param h window height
// 
// Creates the window and defines the quit callback and the expose callback
// To add mouse and key events, <get_window_pointer> function should be used
// Draw a nice cairo background
// Draws a close button
void gloobusWin::create_window(int w, int h)
{
	//g_print ("\nCreating Window...");
	m_w = w;
	m_h = h;

	config.load_config();	

	m_window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );

	GdkScreen * screen = gdk_screen_get_default();
	gtk_widget_set_default_colormap ( gdk_screen_get_rgba_colormap (screen) );
	gtk_window_set_default_size 	( GTK_WINDOW(m_window), m_w, m_h);

	gtk_window_set_position			( GTK_WINDOW (m_window), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_widget_set_app_paintable 	( m_window, 			 TRUE );
	gtk_window_set_decorated 		( GTK_WINDOW (m_window), FALSE);

	gtk_window_set_keep_above       ( GTK_WINDOW (m_window), config.get_ontop());			//Always on top?
	gtk_window_set_skip_taskbar_hint( GTK_WINDOW (m_window), !config.get_taskbar());			//Show in taskbar or not		


	gtk_widget_realize 				( m_window );
	gdk_window_set_back_pixmap 		( m_window->window, NULL, FALSE);			//For the iMpg to not turn it black

    gtk_window_set_icon_name        ( GTK_WINDOW(m_window), "gloobus-preview");

	create_container();				// Creates a gtk_fixed container
	
	//Window destroy Signal

    GdkEventMask event_mask = (GdkEventMask) (gdk_window_get_events (m_window->window) | GDK_POINTER_MOTION_MASK);
    gdk_window_set_events( m_window->window, event_mask);
	g_signal_connect ( GTK_OBJECT(m_window), "destroy", G_CALLBACK(&gloobusWin::quit2) , this);
	g_signal_connect ( m_window, "expose-event",        G_CALLBACK(&gloobusWin::expose_trick), this);
	g_signal_connect ( m_window, "motion_notify_event", G_CALLBACK(&gloobusWin::motion_trick), this);


	if(config.get_focus())
	{
		g_signal_connect ( m_window, "focus-out-event", G_CALLBACK(gtk_main_quit), NULL);
	}
	//g_signal_connect ( m_window, "expose-event", G_CALLBACK(prova), NULL);
	
	create_toolbar();
}

// =============================== CREATE TOOLBAR ============================== //
void gloobusWin::create_toolbar				( void )
{
	GdkScreen * screen = gdk_screen_get_default();
	m_toolbar = gtk_dialog_new_with_buttons	(NULL, GTK_WINDOW(m_window),(GtkDialogFlags)GTK_DIALOG_DESTROY_WITH_PARENT, NULL );	//As a dialog, son of the parent window
	g_object_set(m_toolbar, "has-separator", 0, NULL);
	gtk_widget_set_default_colormap 		(gdk_screen_get_rgba_colormap (screen)  );	//Alpha window
	gtk_window_set_default_size				(GTK_WINDOW(m_toolbar), 480, 100	   	);	//default size
	gtk_window_set_decorated				(GTK_WINDOW(m_toolbar), false		    );	//Don't show gtk buttons
	gtk_window_set_skip_taskbar_hint    	(GTK_WINDOW(m_toolbar), true		    );	//Not show in taskbar
	gtk_widget_set_app_paintable 			(m_toolbar, 			true		    );	//To draw in cairo
	gtk_window_set_keep_above       		(GTK_WINDOW(m_toolbar), true		    );	//Always on top
	
	gtk_widget_realize 						( m_toolbar );
	gdk_window_set_back_pixmap 				( m_toolbar->window, NULL, FALSE);			//For the iMpg to not turn it black

    GdkEventMask event_mask = (GdkEventMask) (gdk_window_get_events (m_toolbar->window) | GDK_POINTER_MOTION_MASK);
    gdk_window_set_events( m_toolbar->window, event_mask);
    g_signal_connect ( m_toolbar, "motion_notify_event", G_CALLBACK(&gloobusWin::motion_trick), this);

	int w = gdk_screen_get_width(screen);
	int h = gdk_screen_get_height(screen);
	gtk_window_move							(GTK_WINDOW(m_toolbar), w/2 - 450/2, h - 120);
	
	//Load the toolbar BG
	//m_toolbar_bg = gdk_pixbuf_new_from_file("file");
	

    gtk_window_set_opacity(GTK_WINDOW(m_toolbar),0);
	g_signal_connect ( m_toolbar, "expose-event", G_CALLBACK(&gloobusWin::toolbar_expose_trick), this);
	//Do not show it, only in fullscreen	
}

//=============================== SHOW TOOLBAR ==================================== //
void gloobusWin::show_toolbar ( void )
{
	GdkScreen * screen = gdk_screen_get_default();
	int w = gdk_screen_get_width	(screen);
	int h = gdk_screen_get_height	(screen);

	gtk_widget_show_all			(m_toolbar);
	gtk_window_move				(GTK_WINDOW(m_toolbar), w/2 - 500/2, h - 100);
	m_toolbarAlpha = 3.0;	//FIXME: This is a bit of a hack to create a false timer.
    m_timeoutID = g_timeout_add (100, &gloobusWin::timeout_trick, this);
}

//=============================== HIDE TOOLBAR ==================================== //
void gloobusWin::hide_toolbar ( void )
{
    g_source_remove(m_timeoutID);
	gtk_widget_hide_all			(m_toolbar);
}
//========================= HIDE TOOLBAR CHECK ==================================== //
bool gloobusWin::hide_toolbar_check(void)
{
    if(m_toolbarAlpha>0)
    {
        m_toolbarAlpha-=0.2;
        gtk_window_set_opacity(GTK_WINDOW(m_toolbar), m_toolbarAlpha);
        return true;
    }
    return false;
}
//=============================== MOTION EVENT ==================================== //
void gloobusWin::motion_event ( void )
{
    //g_debug("Mouse moved\n");
    if(m_toolbarAlpha<=0)
    {
        m_timeoutID = g_timeout_add (100, &gloobusWin::timeout_trick, this);
    }
    m_toolbarAlpha=3.0;	//FIXME: This is a bit of a hack to create a false timer.
    
}
// ============================ GET POINTERS============================= //
GtkWidget*  gloobusWin::get_window_pointer 		( void ){ return m_window;   }
GtkWidget*  gloobusWin::get_container_pointer 	( void ){ return m_container;}
GtkWidget*	gloobusWin::get_toolbar_pointer 	( void ){ return m_toolbar;  }

// ================================== QUIT ====================================== //
// TODO: This functions should not be implemented by window class
void gloobusWin::quit()
{
	
	gtk_main_quit();
}

// =============================== WIN SHOW ALL =============================== //
void gloobusWin::show_all()
{
	gtk_widget_show		(m_window); 		//The last show called
	gtk_widget_show 	(m_container);
}
// ================================ SET TITLE ================================ //
void gloobusWin::set_title (string title )
{
	m_title = title;
	m_title = m_title.substr(m_title.rfind("/")+1,-1);
	
	//Trim to max width
	float 	pixels_per_char = 8.0;
	int 	max = m_title.length();
	int 	width = m_w - SHADOW_WIDTH*2 - 68;	/* -(16+8)*2 = 48 close button, and options button*/
	
	g_debug("Chars %i - Width: %i => Space for = %f",
		max, width , width / pixels_per_char);
	//if(width/max*pixels_per_char  < (int)m_title.length())
	if((int)width/pixels_per_char < (int)m_title.length())
	{	
		m_title = m_title.substr(0,width/pixels_per_char );
		m_title = m_title + "...";	
	}
	gtk_window_set_title (GTK_WINDOW (m_window), m_title.c_str()); 
}
// ================================ SET SIZE ================================ //
void gloobusWin::set_size( int w, int h)
{
	m_w = w;
	m_h = h;
	gtk_window_resize(GTK_WINDOW (m_window),m_w,m_h);
}
// ================================ SET SCREEN SIZE ================================ //
void gloobusWin::set_screen_size( int w, int h)
{
	m_screenW = w;
	m_screenH = h;
}

// ================================== TOOLBAR EXPOSE ================================== //
void gloobusWin::toolbar_expose()
{
	//Clear the background 
	cairo_t *cr;
	cr = gdk_cairo_create 	( m_toolbar->window				);
	cairo_set_source_rgba 	( cr, 0, 0, 0, 0				);
	cairo_set_operator    	( cr, CAIRO_OPERATOR_SOURCE 	);
	cairo_paint	  			( cr 							);
	cairo_set_operator		( cr, CAIRO_OPERATOR_OVER		);
	
	//Draw the SVG background
	//string bgstr = (string)PACKAGE_DATA_PATH + "/floating-bar.png";
	//cairo_surface_t *image = cairo_image_surface_create_from_png(bgstr.c_str()); //NEED TO BE PNG
	if(!m_toolbar_bg) {
		string bgstr = (string)PACKAGE_DATA_PATH + "/floating-bar.png";
		m_toolbar_bg  = cairo_image_surface_create_from_png(bgstr.c_str()); //NEED TO BE PNG
	}
	
	cairo_move_to			( cr, 0, 0);
	//cairo_set_source_surface( cr, image, 0, 0);
	cairo_set_source_surface( cr, m_toolbar_bg, 0, 0);
    cairo_paint				( cr );
	
	//cairo_surface_destroy(image);	//TODO:Make it member variable?
	
	cairo_destroy(cr);
	
}
// ================================== EXPOSE ================================== //
// Expose event, this function is called everytime the window is drawed or redrawed
// It clears everything and draw the backgroun, shadow and header
void gloobusWin::expose()
{
	//g_print("\nDrawing Window");

	cairo_t *cr;

	//Transparent background

	cr = gdk_cairo_create 	( m_window->window);
	cairo_set_source_rgba 	( cr, 0, 0, 0, 0);
	cairo_set_operator    	( cr, CAIRO_OPERATOR_SOURCE );
	cairo_paint	  			( cr );
	cairo_set_operator		( cr, CAIRO_OPERATOR_OVER);

	//Fullscreen?
	GdkScreen *screen = gdk_screen_get_default ();
	gdk_screen_get_width(screen);
	if((m_screenW == m_w) && (m_screenH == m_h))
	{
		draw_fullscreen(cr, m_screenW, m_screenH);
	}
	else	
	{
		//Needs to be first cause it clears the inside!
		draw_shadow		(cr, SHADOW_WIDTH, SHADOW_WIDTH,  m_w-SHADOW_WIDTH*2, m_h-SHADOW_WIDTH*2, WINDOW_RADIOUS);	
	
		draw_header		(cr, SHADOW_WIDTH, SHADOW_WIDTH,  m_w-SHADOW_WIDTH*2, HEADER_HEIGHT, HEADER_RADIOUS);

		draw_bottom		(cr, SHADOW_WIDTH, SHADOW_WIDTH+HEADER_HEIGHT, m_w-SHADOW_WIDTH*2, m_h-HEADER_HEIGHT-SHADOW_WIDTH*2, WINDOW_RADIOUS);
	
		draw_background	(cr, SHADOW_WIDTH, SHADOW_WIDTH+HEADER_HEIGHT, m_w-SHADOW_WIDTH*2, m_h-HEADER_HEIGHT-SHADOW_WIDTH*2, WINDOW_RADIOUS);
		
		draw_title		(cr, m_w);
	}
	
	//cairo_destroy(cr)	;		//Do not uncomment this line, if its uncommented, the widgets in the window are not drawed
	
}


// ==================== DRAW HEADER======================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_header	 	(cairo_t *cr, int x, int y, int w, int h, int r)
{
	cairo_save(cr);
	
	/*
	HEADER
		A ----------------- B
	  F                       C
	  E --------------------- D
	*/ 
	cairo_set_source_rgba (cr,BG_R, BG_G, BG_B, BG_A);		//Header Color
	cairo_move_to(cr,x+r,y) ;                     			//Move to A
	cairo_line_to(cr,x+w-r,y);                    			//Straight line to B
	cairo_curve_to(cr,x+w-r,y,x+w,y,x+w,y+r);   			//Curve to C
	cairo_line_to(cr,x+w,y+h);								//Straight line to D
	cairo_line_to(cr,x,y+h);								//Straight line to E
	cairo_line_to(cr,x,y+r);								//Straight line to F
	cairo_curve_to(cr,x,y+r,x,y,x+r,y);						//Curve to A
	cairo_fill (cr);
	
	//Stroke
	cairo_set_source_rgba (cr, STROKE_R, STROKE_G, STROKE_B, STROKE_A);
	cairo_set_line_width(cr , 1);
	cairo_move_to(cr, x - 0.5, y + h + 0.5);											//Move to E
	cairo_line_to(cr, x - 0.5, y + r + 0.5);											//Line to F
	cairo_curve_to(cr,x - 0.5, y + r + 0.5, x - 0.5 , y + 0.5, x+r - 0.5 ,y + 0.5);		//Curve to A
	cairo_line_to(cr,x+w-r + 0.5,y + 0.5);                  							//Straight line to B
	cairo_curve_to(cr,x+w-r + 0.5,y + 0.5,x+w + 0.5,y + 0.5,x+w + 0.5,y+r + 0.5);   	//Curve to C
	cairo_line_to(cr,x+w + 0.5,y+h + 0.5);												//Straight line to D
	//cairo_line_to(cr,x,y+h);						//Straight line to E
	//cairo_line_to(cr,x,y+r);						//Straight line to F
	//cairo_curve_to(cr,x,y+r,x,y,x+r,y);			//Curve to A
	cairo_stroke (cr);
	
	cairo_restore(cr);
}
// ==================== DRAW BACKGROUND======================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_background (cairo_t *cr, int x, int y, int w, int h, int r)
{
	cairo_save(cr);
	
	h = h -BOTTOM_HEIGHT;

	/*          
	BODY        
	  A ----------------------B
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  D-----------------------C
	*/ 
	cairo_set_source_rgba (cr, BG_R, BG_G, BG_B, BG_A);
	cairo_move_to	( cr,x 		,y );                  	// Move to A
	cairo_line_to	( cr,x+w	,y );                  	//Straight line to B
	cairo_line_to	( cr,x+w	,y+h-r );              	//Straight line to C
	cairo_line_to	( cr,x		,y+h-r );              	//Straight line to D
	cairo_line_to	( cr,x		,y ) ;              	//Straight line to E
	cairo_fill 		( cr );
	
	//Stroke
	cairo_set_source_rgba (cr, STROKE_R, STROKE_G, STROKE_B, STROKE_A);
	cairo_set_line_width(cr , 1);
	
	cairo_move_to	( cr,x - 0.5, y+h-r + 0.5 );                // Move to D
	cairo_line_to	( cr,x - 0.5, y );							// Line to A
	
	cairo_move_to	( cr,x+w + 0.5,y );		                    // Move to B
	cairo_line_to	( cr,x+w + 0.5,y+h-r + 0.5);				// Line to C
	
	cairo_stroke 	( cr );

	cairo_restore(cr);
}

// ==================== DRAW BOTTOM======================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_bottom (cairo_t *cr, int x, int y, int w, int h, int r)
{
	int h2 = h - BOTTOM_HEIGHT;
	
	cairo_save(cr);


	/*  
	BOTTOM
	  A --------------------- B        
	  F                       C
		E ----------------- D
	*/ 
	cairo_set_source_rgba (cr,BG_R, BG_G, BG_B, BG_A);
	cairo_move_to	( cr,x,y+h2-r );                     	// Move to A
	cairo_line_to	( cr,x+w,y+h2-r );                    	// Straight line to B//
	cairo_line_to	( cr,x+w,y+h-r );                    	// Straight line to B//
	cairo_curve_to	( cr,x+w,y+h-r,x+w,y+h,x+w-r,y+h ); 	// Curve to D
	cairo_line_to	( cr,x+r,y+h );                    		// Line to E
	cairo_curve_to	( cr,x+r,y+h,x,y+h,x,y+h-r );      		// Curve to F
	cairo_line_to	( cr,x,y+h2-r  ); 						// Straight to H
	cairo_fill 		( cr );
	
	//Stroke
	cairo_set_source_rgba (cr,  STROKE_R, STROKE_G, STROKE_B, STROKE_A);
	cairo_set_line_width(cr , 1);
	
	cairo_move_to	( cr,x+w + 0.5,y+h2-r + 0.5 );                    						// Move to B
	cairo_line_to	( cr,x+w + 0.5,y+h-r + 0.5 );                    						// Line to C
	cairo_curve_to	( cr,x+w + 0.5,y+h-r + 0.5,x+w + 0.5,y+h + 0.5,x+w-r + 0.5,y+h + 0.5 ); // Curve to D
	cairo_line_to	( cr,x+r - 0.5,y+h + 0.5 );                    							// Line to E
	cairo_curve_to	( cr,x+r - 0.5,y+h + 0.5,x - 0.5,y+h + 0.5,x - 0.5,y+h-r + 0.5 );      	// Curve to F
	cairo_line_to	( cr,x - 0.5,y+h2-r + 0.5  ); 											// Line to A
	
	cairo_stroke 	( cr );
	

	cairo_restore(cr);
}
//==================== DRAW FULLSCREENT ==================== //
// Everything of BG_X color
void gloobusWin::draw_fullscreen(cairo_t *cr, int w, int h)
{
	cairo_save(cr);
	
	int x = 0;
	int y = 0;

	/*          
	BODY        
	  A ----------------------B
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  |                       |
	  D-----------------------C
	*/ 
	cairo_set_source_rgba (cr, BG_R, BG_G, BG_B, BG_A);
	cairo_move_to	( cr,w 		,y );                  	// Move to A
	cairo_line_to	( cr,x+w	,y );                  	//Straight line to B
	cairo_line_to	( cr,x+w	,y+h );              	//Straight line to C
	cairo_line_to	( cr,x		,y+h );              	//Straight line to D
	cairo_line_to	( cr,x		,y ) ;              	//Straight line to E
	cairo_fill 		( cr );
	


	cairo_restore(cr);

}
// ==================== DRAW SHADOW =========================== //
// @param cr Cairo context
// @param x y where the shadow starts
// @param w,h height and widht of the rectangle to draw the shadow
// @param r radious of the borders
void gloobusWin::draw_shadow(cairo_t *cr, int x, int y, int w, int h, int r)
{
	cairo_save(cr);
	cairo_pattern_t* pattern;	

/*
SHADOW

    A ----------------- B
  H                       C
  |                       |
  |                       |
  |                       |
  |                       |
  |                       |
  |                       |
  G                       D
    F ----------------- E
*/ 

	//top left
	int r2 = r/2;
	pattern = cairo_pattern_create_radial	( x+r, y+r, r,x+r, y+r, r+SHADOW_R );
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x-SHADOW_R, y-SHADOW_R, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);

	//top
	pattern = cairo_pattern_create_linear	( 0.0, y-SHADOW_R, 0.0, y);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, 0 );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, SHADOW_A);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x+r, y-SHADOW_R, w-r*2, SHADOW_R);
	cairo_fill 			(cr);

	//top Rigth
	pattern = cairo_pattern_create_radial	( w+x-r, y+r, r, w+x-r, y+r, r+SHADOW_R );
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, w+x-r, y-SHADOW_R, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);
	
	//right
	pattern = cairo_pattern_create_linear	( w+x, 0.0, w+x+SHADOW_R, 0.0);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, w+x, y+r, SHADOW_R, h-r*2);
	cairo_fill 			(cr);
	
		
	//Bottom Rigth
	pattern = cairo_pattern_create_radial	( w+x-r, h+y-r, r, w+x-r, h+y-r, r+SHADOW_R);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, w+x-r, h+y-r, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);
	
	//Bottom
	pattern = cairo_pattern_create_linear	( 0.0, h+y, 0.0, h+y+SHADOW_R);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x+r, h+y, w-r*2, SHADOW_R);
	cairo_fill 			(cr);

	//Bottom Left
	pattern = cairo_pattern_create_radial	( x+r, h+y-r, r, x+r, h+y-r, r+SHADOW_R);
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, SHADOW_A );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, 0 );
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x-SHADOW_R, h+y-r, r+SHADOW_R, r+SHADOW_R);
	cairo_fill 			(cr);

	//Left
	pattern = cairo_pattern_create_linear	( x-SHADOW_R, 0.0, x, 0.0 );
	cairo_pattern_add_color_stop_rgba 		( pattern, 0, 0, 0, 0, 0 );
	cairo_pattern_add_color_stop_rgba 		( pattern, 1, 0, 0, 0, SHADOW_A);
	cairo_set_source 	(cr, pattern);
	cairo_rectangle 	(cr, x-SHADOW_R, y+r, r+SHADOW_R, h-r*2);
	cairo_fill 			(cr);


	//Clear the area inside
	cairo_move_to		( cr, x+r, y);	
	cairo_arc			( cr, x+w-r2, 	y+r2,	r2, PI*1.5, 	PI*2	);
	cairo_arc			( cr, x+w-r, 	y+h-r,	r, 	0,      	PI*0.5	);
	cairo_arc			( cr, x+r,   	y+h-r,	r, 	PI*0.5, 	PI		);
	cairo_arc			( cr, x+r2,  	y+r2, 	r2, PI,     	PI*1.5	);

	cairo_set_operator 	( cr, CAIRO_OPERATOR_CLEAR );
	cairo_fill			( cr );

	cairo_restore(cr);

}

// =================== DRAW TITLE ================================== //
void gloobusWin::draw_title (cairo_t *cr, int w)
{
	// WINDOW TITLE : Notice that the system must have "Lucida Grande" font

	cairo_save				(cr);
	cairo_select_font_face 	(cr, "Lucida Grande", CAIRO_FONT_SLANT_NORMAL,CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size 	(cr, 12.0);
    cairo_text_extents_t extents;
    cairo_text_extents(cr, m_title.c_str(), &extents);

	cairo_move_to 			(cr, w/2 - extents.width/2, SHADOW_WIDTH + HEADER_HEIGHT/2 + 5);
    //g_debug("m_title width: %f\n", extents.width);
    //g_debug("m_title width: %f\n", w/2 - extents.width/2);
    //g_debug("x: %d\n", w/2 - (m_title.length())*7.7/2 );
	cairo_text_path 		(cr, m_title.c_str());
	cairo_set_source_rgba 	(cr, 1, 1, 1 ,1);
	cairo_fill_preserve		(cr);
	cairo_restore			(cr);
	
}

//==================== CREATE CONTAINER ============================ //
void gloobusWin::create_container	( void )
{
	m_container = gtk_fixed_new();								//Create a new container and
	gtk_container_add  (GTK_CONTAINER(m_window),m_container);	//Add it to the main window	
	gtk_widget_show   (m_container);							//This is where the plugins will put his things

}
// ==================== ADD CLOSE BUTTON =========================== //
void gloobusWin::add_close_button	( void )
{
	//g_print("\nAdd Close Button");
	string buttonstr 		= (string)PACKAGE_DATA_PATH + "/window-close.svg";
	//g_debug("Close button: %s\n",buttonstr.c_str());
	m_close_button 			= gtk_image_new_from_file(buttonstr.c_str());	//(string)PACKAGE_LIB_PATH
	GtkWidget *event_box    = gtk_event_box_new();
	
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box),false);
	gtk_container_add	(GTK_CONTAINER(event_box),m_close_button);
    if (config.get_winbar_layout())
    	gtk_fixed_put ( GTK_FIXED(m_container), event_box,
			SHADOW_WIDTH+8, SHADOW_WIDTH + HEADER_HEIGHT/2 - 7);
    else
	    gtk_fixed_put ( GTK_FIXED(m_container), event_box,
			m_w-SHADOW_WIDTH - 23, SHADOW_WIDTH + HEADER_HEIGHT/2 - 7);
	gtk_widget_show_all	(GTK_WIDGET(event_box));
	
	g_signal_connect ( GTK_OBJECT(event_box), "button_press_event",
		G_CALLBACK(gloobus_quit), NULL);

}
// ==================== ADD DRAG BUTTON =========================== //
void gloobusWin::add_drag_button	( void )
{
	/*//g_print("\nAdd drag Button");
	GtkWidget *close_button = gtk_image_new_from_file("/usr/share/gloobus/close_button.png");
	GtkWidget *event_box    = gtk_event_box_new();
	
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box),true);
	gtk_container_add	(GTK_CONTAINER(event_box),close_button);
	gtk_fixed_put		(GTK_FIXED(m_container),event_box, SHADOW_WIDTH+50, SHADOW_WIDTH+2);
	gtk_widget_show_all	(GTK_WIDGET(event_box));
	
	g_signal_connect	(GTK_OBJECT(event_box),"button_press_event", G_CALLBACK(gtk_main_quit), NULL);
	*/

}
// ==================== ADD GLOOBUS BUTTON =========================== //
void gloobusWin::add_gloobus_button	( void )
{
	//g_print("\nAdd Gloobus Button");
	string buttonstr 			= (string)PACKAGE_DATA_PATH + "/gloobus-menu.svg";
	m_gloobus_button 	 		= gtk_image_new_from_file(buttonstr.c_str());	//(string)PACKAGE_LIB_PATH
	GtkWidget *event_box   	 	= gtk_event_box_new();
	
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(event_box),false);
	gtk_container_add	(GTK_CONTAINER(event_box),m_gloobus_button);
    if (config.get_winbar_layout())
	    gtk_fixed_put		(GTK_FIXED(m_container),event_box, m_w-SHADOW_WIDTH - 23, SHADOW_WIDTH + HEADER_HEIGHT/2 - 7);
    else
    	gtk_fixed_put		(GTK_FIXED(m_container),event_box, SHADOW_WIDTH+8, SHADOW_WIDTH + HEADER_HEIGHT/2 - 7);
	gtk_widget_show_all	(GTK_WIDGET(event_box));
	
	g_signal_connect	(GTK_OBJECT(event_box),"button_press_event", G_CALLBACK(&gloobusWin::trick2), this);

}
// ==================== ADD COMMON BUTTONS =========================== //
void gloobusWin::add_common_buttons		( void )
{
	add_close_button();
	add_gloobus_button();
}
// ==================== SHOW COMMON BUTTONS =========================== //
void gloobusWin::show_common_buttons	( void )
{
	gtk_widget_show(m_gloobus_button);
	gtk_widget_show(m_close_button);
}

// ==================== Hide COMMON BUTTONS =========================== //
void gloobusWin::hide_common_buttons	( void )
{
	gtk_widget_hide(m_gloobus_button);
	gtk_widget_hide(m_close_button);
}

// ========================== CREATE ABOUT ============================ //
void gloobusWin::create_about	( void )
{
	static const gchar *authors[] = {
		"Jordi Puigdellívol Hernández <guitarboy000@gmail.com>",
		"ammonkey (aka KitKat) <am.monkeyd@gmail.com>",
		NULL
	};

	static const gchar *artists[] = {
		"Jordi Puigdellívol Hernández <guitarboy000@gmail.com>",
		"Daniel Foré <Daniel.p.Fore@gmail.com>",
		NULL
	};

	string version = "0.4.5\nbzr r"+ (string)REVNO;	//TODO: ADD Version to config.h and change it here!!

	gtk_show_about_dialog(NULL,
						"name", 			"Gloobus Preview",
						"version", 			version.c_str(),
						"copyright", 		"Copyright \xc2\xa9 2009-2010 Gloobus Developers",
						"comments", 		_("See your files in a blink."),
						"authors", 			authors,
						"artists", 			artists,
						"website", 			"http://gloobus.net",
						"website-label", 	"Gloobus.net",
						"logo-icon-name", 	"gloobus-preview",
						NULL);

}
// ========================== SHOW MENU ============================== //
void gloobusWin::show_menu		( void )
{

	GtkWidget *menu 	= gtk_menu_new();
	GtkWidget *settings	= gtk_menu_item_new_with_label(_("Settings"));
	GtkWidget *blog 	= gtk_menu_item_new_with_label(_("Blog"));
	GtkWidget *donate 	= gtk_menu_item_new_with_label(_("Donate"));
	GtkWidget *bug 		= gtk_menu_item_new_with_label(_("Report a Problem"));
	GtkWidget *about	= gtk_menu_item_new_with_label(_("About"));
	GtkWidget *quit 	= gtk_menu_item_new_with_label(_("Quit"));
	

	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),settings);               
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),blog);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),donate);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),bug);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),about);
	gtk_menu_shell_append  (GTK_MENU_SHELL(menu),quit);

	gtk_signal_connect(GTK_OBJECT(settings),"activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(blog),   	"activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(donate), 	"activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(bug),    	"activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(about),  	"activate", G_CALLBACK(&gloobusWin::trick3),  this);
	gtk_signal_connect(GTK_OBJECT(quit),   	"activate", G_CALLBACK(gtk_main_quit), 	NULL);
	
	gtk_widget_show_all(GTK_WIDGET(menu));
	
	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,1,gtk_get_current_event_time());
}

// ==================== LAUNCH MENU ACTION =========================== //
void gloobusWin::launch_menu_action(gchar * label)
{
	string cmd;
	string lb = label;
	if(lb==  _("Settings")) 	cmd = "gloobus-preview-configuration";
	if(lb==  _("Report a Bug")) cmd = "gnome-open \"https://bugs.launchpad.net/gloobus-preview/\"";
	if(lb == _("Blog"))			cmd = "gnome-open \"http://gloobus.wordpress.com\"" ;
	if(lb == _("Donate")) 		cmd = "gnome-open \"https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=guitarboy000%40gmail%2ecom&item_name=Gloobus%2c%20A%20Quicklook%20for%20linux&no_shipping=0&no_note=1&tax=0&currency_code=EUR&lc=US&bn=PP%2dDonationsBF&chars\"" ;
	if(lb == _("About"))	create_about();	

	
	g_spawn_command_line_async(cmd.c_str(), NULL);

}

// =================== GET WIDHT AND GET HEIGHT
int gloobusWin::get_width	( void ){ return m_w;}
int gloobusWin::get_height	( void ){ return m_h;}

void gloobusWin::gloobus_quit(GtkWidget *, gpointer) {
	Gloobus::instance()->quit();
}

void gloobusWin::expose_trick	(GtkWidget *widget, GdkEventExpose *event, gpointer prInstance)						
{
	gloobusWin * instance = (gloobusWin*)prInstance;
	instance->expose( );					
}

bool gloobusWin::toolbar_expose_trick	(GtkWidget *widget, GdkEventExpose *event, gpointer prInstance)						
{
	gloobusWin * instance = (gloobusWin*)prInstance;
	instance->toolbar_expose( );					
	return false;
}

bool gloobusWin::motion_trick	(GtkWidget *widget, GdkEventExpose *event, gpointer prInstance)						
{
	gloobusWin * instance = (gloobusWin*)prInstance;
	instance->motion_event( );						
	return false;
}

gboolean gloobusWin::timeout_trick (gpointer prInstance)				
{
	gloobusWin * instance = (gloobusWin*)prInstance;
	return instance->hide_toolbar_check( );				
}

void gloobusWin::trick2	(GtkWidget *widget, GdkEventExpose *event, gpointer prInstance)						
{
	gloobusWin * instance = (gloobusWin*)prInstance;
	instance->show_menu();				
}

void gloobusWin::trick3	(GtkWidget *widget, GdkEventExpose *event, gpointer prInstance)						
{
	gloobusWin * instance = (gloobusWin*)prInstance;
	instance->launch_menu_action( (gchar*)gtk_menu_item_get_label(GTK_MENU_ITEM(widget)) );		
}
