#ifndef _GLOOBUSWIN_
#define _GLOOBUSWIN_


#include <gtk/gtk.h>
#include <string>
#include <cairo.h>
#include <config.h>

class gloobusWin;

#include "gloobus-preview-defines.h"
#include "gloobus-preview-config.h"
#include "gloobus-preview-singleton.h"


// Class for creating a window, it creates a nice cairo window
// Not so customizable by parameters, just for creating the windows very quickly if I need more than one
// 

using namespace std;

class gloobusWin
{
	private:
		GtkWidget* 	m_window;			//window itself
		GtkWidget* 	m_toolbar;			//Floating toolbar itself
		GtkWidget*	m_container;		//window container
		int			m_screenW;			//Screen Width
		int 		m_screenH;			//Screen Height
		int			m_w;				//window width
		int			m_h;				//window height
		string		m_title;			//window title
		float		m_toolbarAlpha;		//Toolbar Alpha
        int         m_timeoutID;
        cairo_surface_t *m_toolbar_bg;

        gloobusConfig config;

		GtkWidget  *m_gloobus_button;
		GtkWidget  *m_close_button;

	public:
						gloobusWin();
		void 			create_window 			( int w, int h );				// Initializes and creates the window
		void			create_toolbar			( void );
		void			show_toolbar			( void );
		void			hide_toolbar			( void );
		bool			hide_toolbar_check		( void );
		void		 	quit		     		( void );						// Quit (normal)
		static void 	quit2		     		( void ) { gtk_main_quit(); }	// Callback
		void			show_all				( void );						// Show all (Window widget)
		void			set_title				( string title );				// set window title
		void			set_size				( int w, int h);
		void			set_screen_size			( int w, int h);
		GtkWidget*		get_window_pointer 		( void );
		GtkWidget*		get_container_pointer 	( void );
		GtkWidget*		get_toolbar_pointer 	( void );
		void			create_container		( void );
		void			add_common_buttons		( void );
		void			show_common_buttons		( void );
		void			hide_common_buttons		( void );
		int 			get_width				( void );
		int 			get_height				( void );

		//Cairo Drawin functions		
		void			draw_fullscreen		(cairo_t *cr, int w, int h );
		void 			draw_header	 		(cairo_t *cr, int x, int y, int w, int h, int r);
		void 			draw_background 	(cairo_t *cr, int x, int y, int w, int h, int r);
		void 			draw_bottom 		(cairo_t *cr, int x, int y, int w, int h, int r);
		void			draw_shadow			(cairo_t *cr, int x, int y, int w, int h, int r);
		void			draw_title			(cairo_t *cr, int w);
		
	private:
		void			add_close_button	( void );
		void			add_gloobus_button	( void );
		void			add_drag_button		( void );
		void			expose				( void );
		void			toolbar_expose		( void );
		void			motion_event		( void );
		void			show_menu			( void );
		void			launch_menu_action	( gchar * label );
		void			create_about		( void );

		
		static void expose_trick(GtkWidget*, GdkEventExpose*, gpointer);
		static bool toolbar_expose_trick(GtkWidget*, GdkEventExpose*, gpointer);
		static bool motion_trick(GtkWidget*, GdkEventExpose*, gpointer);
        static gboolean timeout_trick(gpointer prInstance);
		static void trick2(GtkWidget*, GdkEventExpose*, gpointer);
		static void trick3(GtkWidget*, GdkEventExpose*, gpointer);
		static void gloobus_quit(GtkWidget *, gpointer);
};

#endif
