#include "plugin-comic.h"

//========================= CONSTRUCTOR ====================//
iComic::iComic()
{
	g_debug("Creating iComic");
	m_n_pages = 0;
}

iComic::~iComic()
{
	g_debug("Destroying iComic");
	m_n_pages = 0;
	g_free(m_extract_command);
	g_free(m_list_command);
}

void iComic::end_plugin(void)
{
	g_debug("Ending iComic");
}

bool iComic::load_doc()
{
	g_debug("Loading iComic");
	gchar 		* std_out = 0;
	gchar		* quoted_file;
	gchar 		**cbr_files;
	gboolean 	success;
	
	
	//1.Depending on mime define list_command and extract_command
	
	//g_print("PLUGIN: Comic file type: %s\n",get_safe_type(false).c_str());
	
	quoted_file = g_shell_quote (m_filename.c_str()); 
	
	if(get_safe_type(false) == "application/x-cbr")
	{
		m_extract_command = g_strdup ("unrar p -c- -ierr");
		m_list_command	  = g_strdup_printf ("unrar vb -c- -- %s", quoted_file); 
	}
	if(get_safe_type(false) == "application/x-cbz")
	{
		m_extract_command = g_strdup ("unzip -p -C");
		m_list_command	  = g_strdup_printf ("zipinfo -1 -- %s", quoted_file); 
	}

	g_free(quoted_file);
	
	//2.Get names of the files in the cbz/cbr sorted! (When they are image files)
	gint exit_status;
	success = g_spawn_command_line_sync (m_list_command, &std_out,
		NULL, &exit_status, NULL);

	if( !exit_status and success and std_out[0] != NULL)
	{
		cbr_files = g_strsplit (std_out, "\n", 0);
		g_free (std_out); 
	
		for (int i = 0; cbr_files[i] != NULL; i++) 
		{ 
			if(check_valid(cbr_files[i]))
			{
				//g_print("PLUGIN: File: %s\n",cbr_files[i]);
				//m_pagenames.push_back(g_strdup (cbr_files[i]));
				m_pagenames.push_back(cbr_files[i]);
				g_free(cbr_files[i]);
				m_n_pages++;
			}
		}
		g_free(cbr_files);
		sort_pagenames();
	}
	else
	{
		g_free (std_out);
		set_error( g_error_new(g_quark_from_string ("iComic"), 201,
			_("Error listing comic book files")) );
		return false;
	}

	get_page_scale();

	return true;
}
//====================== CHECK IF IMAGE ==================== //
bool iComic::check_valid(gchar * filename)
{
	string temp = filename;
	if(temp.find(".jpg")!= string::npos)	return true;
	if(temp.find(".JPG")!= string::npos)	return true;
	if(temp.find(".png")!= string::npos)	return true;
	if(temp.find(".PNG")!= string::npos)	return true;
	
	return false;
}
//====================== GET N PAGES =======================//
int iComic::get_n_pages()
{
	//if(m_n_pages == 0)  load_comic();
	return m_n_pages;
}

//====================== GET PAGES PIXBUF =======================//
GdkPixbuf* iComic::get_page_pixbuf ( int n_page )
{
	GdkPixbuf			* temp;
	GdkPixbufLoader 	* loader; 
	FILE 				* dl;						//PIPE HANDLER
	guchar 				buf[4096];
 	gint 				outpipe = 1;

	

	string cmd = m_extract_command;
	cmd = cmd + " '" + m_filename.c_str() + "' '" + m_pagenames[n_page].c_str()+"'";
	//g_print("PLUGIN: Command %s\n",cmd.c_str());
	
	dl = popen(cmd.c_str(), "r");		//List all plugins (*.so);
   	if(!dl){      
		perror("popen");      
		exit(-1);   	
	}
	loader = gdk_pixbuf_loader_new (); 
	//gdk_pixbuf_loader_set_size(loader, m_width,m_height);
	
	while (outpipe >= 0) 
	{
	 		gssize bytes = fread ( buf, 1, 4096, dl);
 	 		if (bytes > 0) 
	 		{
		 		//g_print("Writing %f bytes\n", (float)bytes);
			 	gdk_pixbuf_loader_write (loader, buf, bytes, NULL);
	 		}
	 		else if (bytes <= 0) 
	 		{
	 			//g_print("Closing Loader\n");
			 	pclose (dl);
	 			gdk_pixbuf_loader_close (loader, NULL);
			 	outpipe = -1;
	 		}
	 	} 


	temp = gdk_pixbuf_loader_get_pixbuf(loader);
	return temp;
}

// ====================== GET WIDTH ============================== //
int iComic::get_page_width( void )
{
	//if(m_n_pages == 0) load_comic();
	return (int)(m_width * m_scale);
}

// ====================== GET HEIGHT ============================== //
int iComic::get_page_height( void )
{
	//if(m_n_pages == 0) load_comic();
	return (int)(m_height * m_scale);
}

// ============================ Calculate size ============================================ //
void iComic::calculate_size(void)
{
	GdkPixbuf	* temp 	= get_page_pixbuf 		( 0 );
	m_height 		 	= gdk_pixbuf_get_height	(temp);
	m_width	 			= gdk_pixbuf_get_width	(temp);
	/*
	int x = get_screen_width()  -150;
	int y = get_screen_height() -150;
	
	if(m_height > y)
	{
		//g_print("Scaling: %d - %d\n",m_height, y);
		m_height = (double)m_height * (double)y/(double)m_height;
		m_width  = (double)m_width  * (double)y/(double)m_height;
	}
	if(m_width > x)
	{
		m_height = (double)m_height * (double)x/(double)m_width;
		m_width  = (double)m_width  * (double)x/(double)m_width;
	}
	*/
	//g_print("PLUGIN: Image Scaled to %ix%i\n",m_width,m_height);
	gdk_pixbuf_unref (temp);
}

// ============================ SORT PAGENAMES ============================================ //
void iComic::sort_pagenames 	( void )
{
	sort( m_pagenames.begin(), m_pagenames.end() );
}

GtkWidget* iComic::get_page(int n) {
	GdkPixbuf * temp = get_page_pixbuf(n);
	GtkWidget * page = gtk_image_new_from_pixbuf(temp);
	g_object_unref(temp);
	return page;
}

void iComic::get_page_scale ( void )
{
	GdkPixbuf * temp = get_page_pixbuf ( 0 );
	
	m_height = gdk_pixbuf_get_height (temp);
	m_width = gdk_pixbuf_get_width (temp);
	gdk_pixbuf_unref (temp);
	
	int sw = get_screen_width() - 150;

	if(m_width <= sw) {
		m_scale = 1;
	} else {
		m_scale = (double)sw / (double)m_width;
	}
}
