#ifndef _ICOMPRESSED_
#define _ICOMPRESSED_

#define MANAGER_VERSION 1				// TODO: Define here the pluginManager version !!! (In pluginManager.h)
#define PLUGIN_NAME	 	"compressed"	// TODO: Define here the plugin Name

#include "../gloobus-preview-interface-list.h"

#include <cstdlib>
#include <csignal>
//#include <stdlib.h>
#include <string>

/*Translations*/
#include <libintl.h>
#include <locale.h>
#define _(String) gettext(String)


#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include <string>
#include <iostream>
#include <algorithm>

#include <gio/gio.h>

#define GP_COMPRESSED_7Z       "application/x-7z-compressed"
#define GP_COMPRESSED_RAR      "application/rar"
#define GP_COMPRESSED_RAR_X    "application/x-rar"
#define GP_COMPRESSED_TAR      "application/x-compressed-tar"
#define GP_COMPRESSED_TAR_7Z   "application/x-7z-compressed-tar"
#define GP_COMPRESSED_TAR_BZIP "application/x-bzip-compressed-tar"
#define GP_COMPRESSED_TAR_LZIP "application/x-lzip-compressed-tar"
#define GP_COMPRESSED_TAR_LZMA "application/x-lzma-compressed-tar"
#define GP_COMPRESSED_TAR_LZOP "application/x-lzop-compressed-tar"
#define GP_COMPRESSED_TAR_X    "application/x-tar"
#define GP_COMPRESSED_TAR_XZ   "application/x-xz-compressed-tar"
#define GP_COMPRESSED_TAR_Z    "application/x-tarz"
#define GP_COMPRESSED_ZIP      "application/zip"

class iCompressed : public iList
{
	private:
		vector<string>		m_files;
		//vector<string>		m_size;
		bool				m_finished;
		GIOChannel		*	m_io_channel;
		vector<string>		cmd_args;
		GPid				cmd_pid;
		bool				m_list_started;
		guint				wait_id;

		bool (iCompressed::*line_parser)( gchar *);
	
	public:
			iCompressed();
			~iCompressed();
		virtual	GdkPixbuf*		get_pixbuf  		( void );
		virtual	GtkListStore*	get_list_store		( void );
		virtual	int				get_n_columns 		( void );
		virtual	string			get_column_title 	( int i);
		virtual	int				get_n_items 		( ) ;
		virtual	string			get_item			( int i, int j ) ;
		virtual	GdkPixbuf*		get_item_pixbuf		( int i, int j) {return NULL;};
		virtual	bool			is_column_pixbuf	( int i);
		virtual	bool			has_items			( void );
				bool				get_archive_files	(  );

		static void wait_child( GPid, gint, gpointer );

	private:
		void add_cmd_arg( string );
		bool exec_cmd( void );

		bool parse_line_7z( gchar * );
		bool parse_line_rar( gchar * );
};



// ============================ Class factories======================================== //
// This is "allways the same" code, just change the plugin name in the printf and the 
// return object
extern "C" iList* create() {		
    return new iCompressed;
}

extern "C" void destroy(iList* p) {
    delete p;
}

extern "C" int getVersion(){			//To save crashes from older versions
	return MANAGER_VERSION;
}

// ============================== REGISTER ============================================ //
// Here is where we register the new plugin, what we do is to add a map with the plugin
// name and the mime file types it can load, then, the pluginManager will take care of wich
// plugin load, depending on its file type. Just add more than one file type adding this
// two lines as many times you want.
//
//	itself.filetype = "image/bmp";			// Add another extension supported
//	pluginVector.push_back(itself);			// Add filetype that plugin reads

extern "C" 
{
	class proxy {
		public:
		proxy() {
			factory_list[PLUGIN_NAME] = create;
			factoryDestroy_list[PLUGIN_NAME] = destroy;

			plugin_filetype itself = {
				PLUGIN_NAME,
				GP_COMPRESSED_7Z,
				"list",
				1
			};
			if(interface::is_program_available("7z"))
				pluginVector.push_back(itself);

			if(interface::is_program_available("unrar")) {
				itself.filetype = GP_COMPRESSED_RAR;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_RAR_X;
				pluginVector.push_back(itself);
			}

			if(interface::is_program_available("tar")) {
				itself.filetype = GP_COMPRESSED_TAR;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_7Z;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_BZIP;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_LZIP;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_LZMA;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_LZOP;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_X;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_XZ;
				pluginVector.push_back(itself);

				itself.filetype = GP_COMPRESSED_TAR_Z;
				pluginVector.push_back(itself);
			}

			if(interface::is_program_available("zipinfo")) {
				itself.filetype = GP_COMPRESSED_ZIP;
				pluginVector.push_back(itself);
			}
		}
	};
	proxy p;	// When we open the dll with RTLD_NOW it creates p, and in consequence, it registers the plugin 
}

#endif

