#include "plugin-gstreamer-audio.h"

//=========================== CONSTRUCTOR ============================//
iMp3::iMp3()
{
	g_debug("Creating iMp3");
	m_created = false;
	
};
iMp3::~iMp3()
{
	g_debug("Destroying iMp3");
	gst_object_unref (GST_OBJECT (m_pipeline));

};

//=========================== GET COVER ============================//
/*GdkPixbuf *get_pix_from_local(string path)
{
    
}*/

GdkPixbuf * iMp3::get_pixbuf ()
{

	g_debug("Loading Mp3 Cover: %s\n",m_filename.c_str());
	GdkPixbuf *temp = NULL;
	
	read_names();							//Read song information

	TagLib::MPEG::File file(m_filename.c_str());
	TagLib::ID3v2::Tag *tagv2 = file.ID3v2Tag();
	if(tagv2)
	{
		TagLib::ID3v2::FrameListMap tags  = tagv2->frameListMap();
		TagLib::ID3v2::FrameList pictures = tags["APIC"];

		
		if(!pictures.isEmpty())				 // Thumbnail exists
		{	
			
		    g_debug("Loading attached Cover\n");				// Just get the front() picture
		    TagLib::ID3v2::AttachedPictureFrame *picture = 
			static_cast<TagLib::ID3v2::AttachedPictureFrame*>(pictures.front());
		    
			TagLib::ByteVector pictureData  = picture->picture();
		   
		    int size   = pictureData.size();
		    char *data = pictureData.data();

			temp = convert_to_pixbuf(data,size);
		}
	}
	// If there is no cover attached, load cover.jpg, then if there is not cover.jpg, try to load
	// cover.png, and there is not cover.png, then, load the default theme icon for audio/mpeg using the
	// function of the parent class "interface"

    string text = m_filename;
    string strf;
    text = m_filename.substr(0,m_filename.rfind('/',m_filename.length()));	// Cut just filename

    const char *cover_filename[3] = {"folder", "cover", 0};
    const char *cover_ext[4] = {"png", "jpg", "jpeg", 0};
    
    for (int i=0; cover_filename[i]; i++)
        for (int j=0; cover_ext[j]; j++)
        {
            strf = text + "/" + string(cover_filename[i]) + "." + string(cover_ext[j]);
            g_debug("Tring to load %s", strf.c_str());
            temp = gdk_pixbuf_new_from_file (strf.c_str(), NULL);
            if (temp != NULL)
                return temp;
        }
	if(temp == NULL)		//If there is no cover.jpg get default icon
	{
		g_warning("No Cover, using default icon\n");
		//temp = get_icon();
		string text2 = (string)PACKAGE_DATA_PATH + "/cover-missing.svg";
		g_debug("Default icon: %s",text2.c_str());
		temp = gdk_pixbuf_new_from_file (text2.c_str(),NULL);
	}

	return temp;
	
}
// ============================== convert_to_pixbuf ============================ //
// Convert a image in memory to string
GdkPixbuf * iMp3::convert_to_pixbuf(char *data,int size)
{
	GdkPixbufLoader * loader = gdk_pixbuf_loader_new();
	gdk_pixbuf_loader_write(loader,(guchar*)data,size,NULL);
	gdk_pixbuf_loader_close(loader,NULL);
	return gdk_pixbuf_loader_get_pixbuf(loader);
}

// ============================== READ NAMES =================================== //
// It loads the values of artits, song and album
void	iMp3::read_names()
{
	//TagLib::MPEG::File file(m_filename.c_str());
	TagLib::FileRef file(m_filename.c_str());
	TagLib::String artist = file.tag()->artist();
	TagLib::String song = 	file.tag()->title(); 
	TagLib::String album = 	file.tag()->album();  
	m_artist =  	artist.toCString(true);
	m_song   =  	song.toCString(true);
	m_album  =  	album.toCString(true);

}
string	iMp3::get_song_artist() { return m_artist;}
string	iMp3::get_song_name	 () { return m_song  ;}
string	iMp3::get_song_album () { return m_album ;}


// ================================== PLAY =================================== //
void 	iMp3::play	(void)
{
	if(m_created == false)
	{
		g_debug("Start Playing...\n");
		gst_init (NULL, NULL);
	
		m_pipeline = gst_element_factory_make ("playbin", "player");
		g_object_set (G_OBJECT (m_pipeline), "uri", g_file_get_uri ( m_gfile), NULL);
	
		{
			GstBus *bus;

			bus = gst_pipeline_get_bus (GST_PIPELINE (m_pipeline));
			gst_object_unref (bus);

		}
		m_created = true;
	}

	gst_element_set_state (GST_ELEMENT (m_pipeline), GST_STATE_PLAYING);

}

// ================================== PAUSE =================================== //
void 	iMp3::pause	(void)
{
	gst_element_set_state (GST_ELEMENT (m_pipeline), GST_STATE_PAUSED);
}

// ================================== STOP =================================== //
void 	iMp3::stop	(void)
{
	gst_element_set_state (GST_ELEMENT (m_pipeline), GST_STATE_READY);
}

// ================================== SEEK =================================== //
// Move the pipeline to the value (value range is from 0 to 100) so
// a division with the duration must be done
void iMp3::seek	( double value )
{
	//if(!DURATION_IS_VALID (m_duration))
	 m_duration = get_int_duration();
		
	 if (DURATION_IS_VALID (m_duration))
	 {	 
		 
		GstSeekFlags seek_flags = GstSeekFlags(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT);
		guint64 to_seek = m_duration  * value / 100;
		//guint64 to_seek = 13899408005078;
		//g_print("PLUGIN: SEEK to %d (%d)\n",value,to_seek);
		//g_print("PLUGIN: SEEK to %.2f -Seek: %llu - Duration: %llu\n",value,to_seek,m_duration);

		gst_element_seek (m_pipeline, 1.0,
				          GST_FORMAT_TIME,
				          GST_SEEK_FLAG_FLUSH,
				          GST_SEEK_TYPE_SET, to_seek,
				          GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
		
    }
    else
    {
    	g_critical("Invalid Duration\n");
    }
}


// =============================== GET INT DURATION ============================= //
guint64 iMp3::get_int_duration( void )
{

    GstFormat format = GST_FORMAT_TIME;
    gint64 cur;
    gboolean result;

    result = gst_element_query_duration (m_pipeline, &format, &cur);
    if (!result || format != GST_FORMAT_TIME)
        return GST_CLOCK_TIME_NONE;

    return cur;
}
// =============================== GET POSITION ============================= //
double iMp3::get_position	( void )
{
	m_duration = get_int_duration();	
    GstFormat format = GST_FORMAT_TIME;
    gint64 cur;
    gboolean result;

    result = gst_element_query_position (m_pipeline, &format, &cur);
    if (!result || format != GST_FORMAT_TIME)
        return 0;

	m_current_time = cur;
    return  cur * (((double) 100) / m_duration);

}
// ================================= GET DURATION (string) ============================ //
string iMp3::get_duration	( void )
{
	if (DURATION_IS_VALID (m_duration))
	{
		return convert_seconds(m_duration);
	}	
	else 
		return "--:--";
}

// ================================= GET CURRENT TIME (string) ============================ //
string iMp3::get_current_time	( void )
{	
	if (DURATION_IS_VALID (m_duration))
	{
		return convert_seconds(m_current_time);
	}
	
	else 
		return "--:--";
}

// ==================== CONVERT SECONDS =================== //
string iMp3::convert_seconds(gint64 longSeconds)
{
	gint64 new_sec = longSeconds  / 1000;
	new_sec = new_sec / 1000;
	new_sec = new_sec / 1000;

	char d[6];
	int minutes = (int)new_sec/60;
	int seconds = (int)new_sec%60;

	sprintf(d,"%02i:%02i",minutes,seconds);

		return d;
}
