#include "plugin-gstreamer-video.h"

iMpg::iMpg()
{
	g_debug("Creating iMpg");
	m_created = false;
}
iMpg::~iMpg()
{
	g_debug("Destroy iMpg");
	//gst_object_unref (GST_OBJECT (m_pipeline));
	gst_object_unref (GST_OBJECT (m_bin));
}

extern "C" {
static void gst_notify_stream_info_callback(GstElement* element,
                                            GParamSpec* pspec,
                                            iMpg* be)
{
    g_debug("gst_notify_stream_info_callback");
    be->QueryVideoSizeFromElement(be->m_bin);
}
}

extern "C" {
static void gst_notify_caps_callback(GstPad* pad,
                                     GParamSpec* pspec,
                                     iMpg* be)
{
    g_debug("gst_notify_caps_callback");
    be->QueryVideoSizeFromPad(pad);
}
}


bool iMpg::QueryVideoSizeFromPad(GstPad* pad)
{
    const GstCaps* caps = GST_PAD_CAPS(pad);
    if ( caps )
    {
        const GstStructure *s = gst_caps_get_structure (caps, 0);
        //wxASSERT(s);

        gst_structure_get_int (s, "width", &m_videoSize_width);
        gst_structure_get_int (s, "height", &m_videoSize_height);

        const GValue *par;
        par = gst_structure_get_value (s, "pixel-aspect-ratio");

        if (par)
        {
            g_debug("pixel-aspect-ratio found in pad");
            int num = par->data[0].v_int,
                den = par->data[1].v_int;

            // TODO: maybe better fraction normalization...
            if (num > den)
                m_videoSize_width = (int) ((float) num * m_videoSize_width / den);
            else
                m_videoSize_height = (int) ((float) den * m_videoSize_height / num);
        }

        g_debug("Adjusted video size: [%d,%d]", m_videoSize_width, m_videoSize_height);
        return true;
    } // end if caps

    return false; // not ready/massive failure
}

bool iMpg::QueryVideoSizeFromElement(GstElement* element)
{
    const GList *list = NULL;
    g_object_get (G_OBJECT (element), "stream-info", &list, NULL);

    for ( ; list != NULL; list = list->next)
    {
        GObject *info = (GObject *) list->data;
        gint type;
        GParamSpec *pspec;
        GEnumValue *val;
        GstPad *pad = NULL;

        g_object_get (info, "type", &type, NULL);
        pspec = g_object_class_find_property (
                        G_OBJECT_GET_CLASS (info), "type");
        val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);

        if (!strncasecmp(val->value_name, "video", 5) ||
            !strncmp(val->value_name, "GST_STREAM_TYPE_VIDEO", 21))
        {
            // Newer gstreamer 0.8+ plugins are SUPPOSED to have "object"...
            // but a lot of old plugins still use "pad" :)
            pspec = g_object_class_find_property (
                        G_OBJECT_GET_CLASS (info), "object");

            if (!pspec)
                g_object_get (info, "pad", &pad, NULL);
            else
                g_object_get (info, "object", &pad, NULL);

            if(!QueryVideoSizeFromPad(pad))
            {
                // wait for those caps to get ready
                g_signal_connect(pad, "notify::caps", G_CALLBACK(gst_notify_caps_callback), this);
            }
            break;
        }// end if video
    }// end searching through info list

    // no video (or extremely delayed stream-info)
    if(list == NULL)
    {
        g_debug("no video?");
        //m_videoSize = wxSize(0,0);
        return false;
    }

    return true;
}


// ================================ LOAD ============================== //
void iMpg::play()
{
	if(m_created == false)
	{
		g_debug("Start Playing (%s)",g_file_get_uri ( m_gfile));

		gst_init (NULL, NULL);

		//m_pipeline	= gst_pipeline_new 		   ("gst-player");
		m_bin 		= gst_element_factory_make ("playbin", 		"m_bin");
		m_sink 		= gst_element_factory_make ("xvimagesink", 	"m_sink");
		g_object_set (G_OBJECT (m_bin), "video-sink", m_sink, NULL);
		//gst_bin_add  (GST_BIN (m_pipeline), m_bin);


		//GstBus *bus = gst_pipeline_get_bus (GST_PIPELINE (m_pipeline));
		GstBus *bus = gst_pipeline_get_bus (GST_PIPELINE (m_bin));
		//gst_bus_add_watch(bus, playbin_bus_callback, &m_bin);
		gst_object_unref (bus);
		g_signal_connect(m_bin, "notify::stream-info",
		                 G_CALLBACK(gst_notify_stream_info_callback), this);
	
		g_object_set (G_OBJECT (m_bin), "uri", g_file_get_uri( m_gfile), NULL);
		g_object_set (G_OBJECT (m_sink), "force-aspect-ratio", TRUE, NULL);
		//g_object_set (G_OBJECT (m_sink), "draw-borders", FALSE, NULL);
			
		if (GST_IS_X_OVERLAY (m_sink))
		{
			//printf("Is Overlay (Win ID: %i)!!\n",GPOINTER_TO_INT (m_window));
	
			gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (m_sink),GDK_WINDOW_XWINDOW (m_drawable->window));
			gst_x_overlay_handle_events  (GST_X_OVERLAY (m_sink), FALSE);
		}

		// getting video dimension informations
		//gst_element_set_state (m_pipeline, GST_STATE_PAUSED);
		//gst_element_set_state (m_bin, GST_STATE_PAUSED);
		
		//if (GstPad* pad = gst_element_get_static_pad(m_pipeline, "gst-player"))
		/*if (GstPad* pad = gst_element_get_static_pad(m_bin, "playbin"))
		{
		    gst_video_get_size(GST_PAD(pad), &width, &height);
		    //gst_object_unref(GST_OBJECT(pad));
		    g_debug("VVV Video %d %d\n", width, height);
		}
		else
		    g_debug("VVV Video failed\n");
	*/
		/*GstElementFactory *factory = gst_element_factory_find ("xvimagesink");
		g_print ("The '%s' element is a member of the category %s.\n"
		       "Description: %s\n",
		       gst_plugin_feature_get_name (GST_PLUGIN_FEATURE (factory)),
		       gst_element_factory_get_klass (factory),
		       gst_element_factory_get_description (factory));*/


	/* 
		GList *streaminfo = NULL;
		GstPad *audiopad = NULL;
		GstPad *videopad = NULL;

		g_object_get (G_OBJECT(m_bin), "stream-info", &streaminfo, NULL);
		//g_object_get (m_pipeline, "stream-info", &streaminfo, NULL);
		if (streaminfo != NULL)
		    g_debug("grrrrrrrrrrrrrrrrrrrrrr\n");

		for (int i=0; streaminfo; streaminfo = streaminfo->next, i++) 
		{
		    g_debug("loop %i\n", i);
		    GObject *info;
		    int type;
		    GParamSpec *pspec;
		    GEnumValue *val;

		    info = (GObject *) streaminfo->data;
		    if (info == NULL)
		        continue;
		    type = -1;
		    g_object_get (info, "type", &type, NULL);
		    pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
		    val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);

		    if (strcmp (val->value_nick, "audio") == 0) 
		    {
		        g_debug("has_audio\n");
		        if (audiopad == NULL)
		            g_object_get (info, "object", &audiopad, NULL);
		    }
		    else if (strcmp (val->value_nick, "video") == 0)
		    {
		        g_debug("has_video\n");
		        if (videopad == NULL)
		            g_object_get (info, "object", &videopad, NULL);
		    }
		}
	*/
		m_created = true;
	}
	
	gst_element_set_state (m_bin, GST_STATE_PLAYING);
	
}
// =============================== STOP ============================= //
void iMpg::pause()
{
	//gst_element_set_state (GST_ELEMENT (m_pipeline), GST_STATE_READY);
	gst_element_set_state (GST_ELEMENT (m_bin), GST_STATE_PAUSED);

}
// =============================== STOP ============================= //
void iMpg::stop()
{
	//gst_element_set_state (GST_ELEMENT (m_pipeline), GST_STATE_READY);
	gst_element_set_state (GST_ELEMENT (m_bin), GST_STATE_READY);

}

// ================================== SEEK =================================== //
// Move the pipeline to the value (value range is from 0 to 100) so
// a division with the duration must be done
void iMpg::seek	( double value )
{
	//if(!DURATION_IS_VALID (m_duration))
		m_duration = get_int_duration();
		
	 if (DURATION_IS_VALID (m_duration))
	 {	 
		 
		GstSeekFlags seek_flags = GstSeekFlags(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT);
		guint64 to_seek = m_duration  * value / 100;
		//guint64 to_seek = 13899408005078;
		//g_print("PLUGIN: SEEK to %d (%d)\n",value,to_seek);
		//g_print("PLUGIN: SEEK to %.2f -Seek: %llu - Duration: %llu\n",value,to_seek,m_duration);

		//gst_element_seek (m_pipeline, 1.0,
		gst_element_seek (m_bin, 1.0,
				          GST_FORMAT_TIME,
				          GST_SEEK_FLAG_FLUSH,
				          GST_SEEK_TYPE_SET, to_seek,
				          GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
    }
    else
    {
    	g_critical("Invalid Duration");
    }
}


// =============================== GET  INT DURATION ============================= //
guint64 iMpg::get_int_duration( void )
{

    GstFormat format = GST_FORMAT_TIME;
    gint64 cur;
    gboolean result;

    //result = gst_element_query_duration (m_pipeline, &format, &cur);
    result = gst_element_query_duration (m_bin, &format, &cur);
    if (!result || format != GST_FORMAT_TIME)
        return GST_CLOCK_TIME_NONE;

    return cur;
}
// =============================== GET POSITION ============================= //
double iMpg::get_position	( void )
{
	m_duration = get_int_duration();
    GstFormat format = GST_FORMAT_TIME;
    gint64 cur;
    gboolean result;

    //result = gst_element_query_position (m_pipeline, &format, &cur);
    result = gst_element_query_position (m_bin, &format, &cur);
    if (!result || format != GST_FORMAT_TIME)
        return 0;

	m_current_time = cur;
    return  cur * (((double) 100) / m_duration);

}
// ================================= GET DURATION (string) ============================ //
string iMpg::get_duration	( void )
{
	if (DURATION_IS_VALID (m_duration))
	{
		return convert_seconds(m_duration);
	}	
	else 
		return "--:--";
}
// ================================= GET CURRENT TIME (string) ============================ //
string iMpg::get_current_time	( void )
{	
	if (DURATION_IS_VALID (m_duration))
	{
		return convert_seconds(m_current_time);
	}
	
	else 
		return "--:--";
}
// ==================== CONVERT SECONDS =================== //
string iMpg::convert_seconds(gint64 longSeconds)
{
        gint64 new_sec = longSeconds  / 1000;
        new_sec = new_sec / 1000;
        new_sec = new_sec / 1000;

        char d[9];
        int minutes = (int)new_sec/60;
        int seconds = (int)new_sec%60;
       
        if ( minutes/60 > 0 )
        {
			int hours = (int)minutes/60;
			minutes = (int)minutes%60;
			sprintf(d,"%02i:%02i:%02i",hours,minutes,seconds);
                
        }
        else
        {
			sprintf(d,"%02i:%02i",minutes,seconds);
        }

        return d;
}
