#include "plugin-pdf.h"

#define WIDTH_PERCENT 0.8
//======================= CONSTRUCTOR ===================== //
iPdf::iPdf(void)
	:m_document(0)
	,m_page(0)
	,m_scale(1.0)
{
	g_debug("Creating iPdf");
}

iPdf::~iPdf(void)
{
	g_debug("Destroying iPdf");
	if(m_document)
		g_object_unref(m_document);
	if(m_page)
		g_object_unref(m_page);
}

void iPdf::end_plugin(void)
{
	g_debug("Ending iPdf");
}
//====================== LOAD ==============================//
bool iPdf::load_doc()
{
	//char * uri = g_file_get_uri ( m_gfile);
	char * uri = get_pdf_uri();
	if(!uri)
		return false;
	g_debug("Temp URI: %s", uri);
	m_document = poppler_document_new_from_file (uri, NULL, NULL);
	if ( !m_document ) {
		set_error( g_error_new(g_quark_from_string ("iPdf"), 501,
			_("Error loading PDF file")) );
		return false;
	}

	m_page = poppler_document_get_page (m_document, 0);
	if ( !m_page ) {
		set_error( g_error_new(g_quark_from_string ("iPdf"), 502,
			_("Error loading first page")) );
		return false;
	}

	get_scale();
	g_free(uri);
	return true;
}
//====================== GET N PAGES =======================//
int iPdf::get_n_pages()
{
	return  poppler_document_get_n_pages(m_document);
}

//====================== GET PAGES WIDTH =======================//
int iPdf::get_page_width_unscaled  ( void )
{
	double x,y;
	poppler_page_get_size (m_page, &x, &y);
	return x;	

}
//====================== GET PAGES HEIGHT =======================//
int	iPdf::get_page_height_unscaled ( void )
{
	double x,y;
	poppler_page_get_size (m_page, &x, &y);
	return y;	
}
//====================== GET PAGE PIXBUF =======================//
GdkPixbuf* iPdf::get_page_pixbuf ( int n_page )
{
	GdkPixbuf* temp;
	if(m_page)
		g_object_unref(m_page);
	m_page = poppler_document_get_page 	( m_document, n_page);
	temp = gdk_pixbuf_new( GDK_COLORSPACE_RGB, FALSE, 8, 
		(int)(get_page_width_unscaled() * m_scale), 
		(int)(get_page_height_unscaled() * m_scale));
	poppler_page_render_to_pixbuf ( m_page,0, 0, 
		(int)(get_page_width_unscaled() * m_scale),
		(int)(get_page_height_unscaled() * m_scale),
		m_scale, 0, temp);
	return temp;
}


// ====================== GET SCALE ============================== //
void iPdf::get_scale( void )
{
	int x = get_screen_width()-150;
	int y = get_screen_height()-150;

	int x2 = get_page_width_unscaled();
	int y2 = get_page_height_unscaled();

	//g_debug("MIDES:%dx%d - %dx%d\n", x,y,x2,y2);

	m_scale = x*WIDTH_PERCENT / x2;
	
}
// ====================== GET HEIGHT ============================== //
int iPdf::get_page_height( void ) {
	int height = (int)(get_page_height_unscaled() * m_scale);
	g_debug("PAGE HEIGHT SCALED: %d\n",height);
	return height;
}
// ====================== GET HEIGHT ============================== //
int iPdf::get_page_width( void )
{
	g_debug("PAGE WIDTH SCALED: %d\n",(int)(get_page_width_unscaled() * m_scale));
	return (int)(get_page_width_unscaled() * m_scale);
	//return 800;
}

GtkWidget* iPdf::get_page(int n) {
	GdkPixbuf * temp = get_page_pixbuf(n);
	GtkWidget * page = gtk_image_new_from_pixbuf(temp);
	g_object_unref(temp);
	return page;
}

char * iPdf::get_pdf_uri( void ) {
	string type = get_safe_type(false);
	if(type == "application/pdf") {
		return g_file_get_uri ( m_gfile);
	} else {
		g_debug("Trying to convert to PDF");
		string tmp = get_temp_path();
		if(g_file_test(tmp.c_str(),G_FILE_TEST_EXISTS))
			return path_to_uri(tmp);

		//if( !check_file_size()) {
		if(get_file_size(m_filename) > GP_IDOC_MAX_SIZE) {
			set_error( g_error_new(g_quark_from_string ("iPdf"), 512,
				_("File is too big for convertion to PDF")) );
			return NULL;
		}
		
		string cmd;
		if( interface::is_program_available("abiword") &&
			(  type == "application/vnd.oasis.opendocument.text"
			|| type == "application/msword"
			|| type == "application/rtf"
			) )
		{
			cmd = "abiword -t pdf -o \""+tmp+"\" \""+m_filename+"\"";
			int result = system(cmd.c_str());
			if( result == 0)
				return path_to_uri(tmp);
		}

		if( interface::is_program_available("unoconv") ) {
			cmd = "unoconv --stdout \"" + m_filename + "\" > \""+tmp+"\"";
			int result = system(cmd.c_str());
			if( result == 0)
				return path_to_uri(tmp);
		}
		set_error( g_error_new(g_quark_from_string ("iPdf"), 511,
			_("Error converting file to PDF")) );
	}
	return NULL;
}

string iPdf::get_temp_path( void ) {
	char * temp = g_file_get_path(m_gfile);
	string path(temp);
	g_free(temp);
	char * hash = g_compute_checksum_for_data(G_CHECKSUM_SHA256,
		(const guchar*)path.c_str(), path.size());
	string result = string("/tmp/gloobus.") + string(hash) + string(".pdf");
	g_free(hash);
	g_debug("Temp path: %s", result.c_str());
	return result;
}

char * iPdf::path_to_uri(string path) {
	path = "file://" + path;
	return g_uri_escape_string(path.c_str(),
		G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, false);
}
