#include "plugin-ps.h"

iPs::iPs(void)
	:p_document(0)
	,p_page_width(0)
	,p_page_height(0)
	,p_page_scale(1.0)
{
	g_debug("Creating iPs");
}

iPs::~iPs(void)
{
	g_debug("Destorying iPs");
	spectre_document_free(p_document);
}

void iPs::end_plugin(void)
{
	g_debug("Ending iPs");
}
//====================== LOAD ==============================//
bool iPs::load_doc()
{
	gchar *filename;
	GError **error = NULL;
	
	filename = g_filename_from_uri (g_file_get_uri(m_gfile), NULL, error);
	if (!filename) {
		set_error( g_error_new(g_quark_from_string ("iPs"), 401,
			_("Error getting filename")) );
		if(*error)
			g_error_free(*error);
		return false;
	}
	
	p_document = spectre_document_new ();
	spectre_document_load (p_document, filename);
	if (spectre_document_status (p_document)) {
		set_error( g_error_new(g_quark_from_string ("iPs"), 402,
			_("Failed to open document")) );
		g_free (filename);
		return false;
	}
	g_free (filename);
	get_page_scale();

	return true;
}

int iPs::get_n_pages()
{
	return spectre_document_get_n_pages (p_document);
}

void iPs::get_page_size (gint page, int *width, int *height)
{
	SpectrePage *ps_page;
	ps_page = spectre_document_get_page (p_document, page);
	spectre_page_get_size (ps_page, width, height);
	spectre_page_free(ps_page);
}


//====================== GET PAGES WIDTH =======================//
int iPs::get_page_width_unscaled  ( void )
{
	return p_page_width;
}
//====================== GET PAGES HEIGHT =======================//
int	iPs::get_page_height_unscaled ( void )
{
	return p_page_height;
}
//====================== GET PAGE PIXBUF =======================//
GdkPixbuf* iPs::get_page_pixbuf ( int n_page )
{
	GdkPixbuf            *pixbuf;
	SpectrePage          *ps_page;
	SpectreRenderContext *src;
	gint                  width_points;
	gint                  height_points;
	gint                  width, height;
	guchar               *data = NULL;
	gint                  stride;

	ps_page = spectre_document_get_page (p_document, n_page);

	spectre_page_get_size (ps_page, &width_points, &height_points);
	width = (int)(width_points * p_page_scale);
	height = (int)(height_points * p_page_scale);

	src = spectre_render_context_new ();
	spectre_render_context_set_scale (src,
		(gdouble)width / width_points,
		(gdouble)height / height_points);

	spectre_page_render (ps_page, src, &data, &stride);
	spectre_render_context_free (src);
	if (!data) {
		g_critical("iPs: failed to render page\n");
		exit(-1);
	}
	if (spectre_page_status (ps_page)) {
		g_warning ("%s", spectre_status_to_string (spectre_page_status (ps_page)));
		g_free (data);
		exit(-1);
	}
	
	spectre_page_free(ps_page);

	for(int i=3; i < height*stride ; i=i+4) {
		data[i] = 255;
	}

	pixbuf = gdk_pixbuf_new_from_data(data, GDK_COLORSPACE_RGB, true, 8,
		(int)width, (int)height, stride, ps_free, NULL);

	return pixbuf;
}

void iPs::get_page_scale ( void )
{
	get_page_size (0, &p_page_width, &p_page_height);
	
	int sw = get_screen_width() - 150;

	if(p_page_width <= sw) {
		p_page_scale = 1;
	} else {
		p_page_scale = (double)sw / (double)p_page_width;
	}
}

int iPs::get_page_width( void )
{
	return (int)(p_page_width * p_page_scale);
}

int iPs::get_page_height( void )
{
	return (int)(p_page_height * p_page_scale);
}

GtkWidget* iPs::get_page(int n) {
	GdkPixbuf * temp = get_page_pixbuf(n);
	GtkWidget * page = gtk_image_new_from_pixbuf(temp);
	g_object_unref(temp);
	return page;
}

void iPs::ps_free(guchar* data, void* ptr) {
	g_free(data);
}
